@@ -1,5 +1,5 @@
 /*
- * $Id: conf.c 1364 2006-08-26 03:14:48Z rpedde $
+ * $Id: conf.c 1690 2007-10-23 04:23:50Z rpedde $
  * Functions for reading and writing the config file
  *
  * Copyright (C) 2006 Ron Pedde (ron@pedde.com)
@@ -48,26 +48,29 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "daapd.h"
 #include "conf.h"
 #include "err.h"
 #include "ll.h"
-#include "daapd.h"
 #include "os.h"
+#include "util.h"
 #include "webserver.h"
 #include "xml-rpc.h"
 
+#include "io.h"
 
 #ifndef HOST_NAME_MAX
 # define HOST_NAME_MAX 255
 #endif
 
+#define MAX_REND_LEN 62
+
 /** Globals */
 //static int ecode;
 static LL_HANDLE conf_main=NULL;
 static LL_HANDLE conf_comments=NULL;
 
 static char *conf_main_file = NULL;
-static pthread_mutex_t conf_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 #define CONF_LINEBUFFER 1024
 
@@ -85,20 +88,24 @@ typedef struct _CONF_ELEMENTS {
     char *term;
 } CONF_ELEMENTS;
 
+typedef struct _CONF_MAP {
+    char *old_section;
+    char *old_key;
+    char *new_section;
+    char *new_key;
+} CONF_MAP;
+
 /** Forwards */
 static int _conf_verify(LL_HANDLE pll);
 static LL_ITEM *_conf_fetch_item(LL_HANDLE pll, char *section, char *key);
 static int _conf_exists(LL_HANDLE pll, char *section, char *key);
-static void _conf_lock(void);
-static void _conf_unlock(void);
-static int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent);
+static int _conf_write(IOHANDLE hfile, LL *pll, int sublevel, char *parent);
 static CONF_ELEMENTS *_conf_get_keyinfo(char *section, char *key);
 static int _conf_makedir(char *path, char *user);
 static int _conf_existdir(char *path);
-static int _conf_split(char *s, char *delimiters, char ***argvp);
-static void _conf_dispose_split(char **argv);
 static int _conf_xml_dump(XMLSTRUCT *pxml,LL *pll,int sublevel,char *parent);
 static int _conf_verify_element(char *section, char *key, char *value);
+static void _conf_remap_entry(char *old_section, char *old_key, char **new_section, char **new_key);
 
 static CONF_ELEMENTS conf_elements[] = {
     { 1, 0, CONF_T_STRING,"general","runas" },
@@ -117,14 +124,16 @@ static CONF_ELEMENTS conf_elements[] = {
     { 0, 0, CONF_T_INT,"general","process_m3u" },
     { 0, 0, CONF_T_INT,"general","scan_type" },
     { 0, 1, CONF_T_INT,"general","compress" },
-    { 0, 0, CONF_T_STRING,"general","playlist" },
+    { 0, 1, CONF_T_STRING,"general","playlist" },
     { 0, 0, CONF_T_STRING,"general","extensions" },
     { 0, 0, CONF_T_STRING,"general","interface" },
     { 0, 0, CONF_T_STRING,"general","ssc_codectypes" },
     { 0, 0, CONF_T_STRING,"general","ssc_prog" },
     { 0, 0, CONF_T_STRING,"general","password" },
+    { 0, 0, CONF_T_STRING,"general","never_transcode" },
     { 0, 0, CONF_T_MULTICOMMA,"general","compdirs" },
     { 0, 0, CONF_T_STRING,"general","logfile" },
+    { 0, 0, CONF_T_INT,"general","truncate" },
     { 0, 0, CONF_T_EXISTPATH,"plugins","plugin_dir" },
     { 0, 0, CONF_T_MULTICOMMA,"plugins","plugins" },
     { 0, 0, CONF_T_INT,"daap","empty_strings" },
@@ -135,10 +144,53 @@ static CONF_ELEMENTS conf_elements[] = {
     { 0, 0, CONF_T_INT,"scanning","ignore_dotfiles" },
     { 0, 0, CONF_T_INT,"scanning","concat_compilations" },
     { 0, 0, CONF_T_INT,"scanning","case_sensitive" },
+    { 0, 0, CONF_T_INT,"scanning","follow_symlinks" },
+    { 0, 0, CONF_T_INT,"scanning","skip_first" },
+    { 0, 0, CONF_T_STRING,"scanning","mp3_tag_codepage" },
     { 0, 0, CONF_T_INT,"scan","correct_order" },
-    { 0, 0, CONF_T_INT, NULL, NULL }
+
+    /* remapped values */
+    { 0, 0, CONF_T_INT,"scanning","process_playlists" },
+    { 0, 0, CONF_T_INT,"scanning","process_itunes" },
+    { 0, 0, CONF_T_INT,"scanning","process_m3u" },
+    { 0, 0, CONF_T_INT,"daap","correct_order" },
+    { 0, 0, CONF_T_INT, NULL, NULL },
 };
 
+static CONF_MAP _conf_map[] = {
+    { "general","process_m3u","scanning","process_playlists" },
+    { "scanning","process_xml","scanning","process_itunes" },
+    { "scan","correct_order","daap","correct_order" },
+
+    { NULL, NULL, NULL, NULL }
+};
+
+/**
+ * Upgrade a config silently by remapping old configuration values
+ * to new configuration values
+ */
+void _conf_remap_entry(char *old_section, char *old_key, char **new_section, char **new_key) {
+    CONF_MAP *pmap;
+
+    pmap = _conf_map;
+    while(pmap->old_section) {
+        if((!strcasecmp(old_section,pmap->old_section)) &&
+           (!strcasecmp(old_key,pmap->old_key))) {
+            /* found it! */
+            DPRINTF(E_LOG,L_CONF,"Config option %s/%s has "
+                    "been replaced by %s/%s\n",
+                    pmap->old_section,pmap->old_key,
+                    pmap->new_section, pmap->new_key);
+            *new_section = pmap->new_section;
+            *new_key = pmap->new_key;
+            return;
+        }
+        pmap++;
+    }
+
+    *new_section = old_section;
+    *new_key = old_key;
+}
 
 /**
  * Try and create a directory, including parents (probably
@@ -241,30 +293,6 @@ CONF_ELEMENTS *_conf_get_keyinfo(char *section, char *key) {
     return found ? pcurrent : NULL;
 }
 
-/**
- * lock the conf mutex
- */
-void _conf_lock() {
-    int err;
-
-    if((err=pthread_mutex_lock(&conf_mutex))) {
-        DPRINTF(E_FATAL,L_CONF,"Cannot lock configuration mutex: %s\n",
-            strerror(err));
-    }
-}
-
-/**
- * unlock the conf mutex
- */
-void _conf_unlock() {
-    int err;
-
-    if((err = pthread_mutex_unlock(&conf_mutex))) {
-        DPRINTF(E_FATAL,L_CONF,"Cannot unlock configuration mutex %s\n",
-            strerror(err));
-    }
-}
-
 /**
  * fetch item based on section/term basis, rather than just a single
  * level deep, like ll_fetch_item does
@@ -346,18 +374,18 @@ int _conf_verify_element(char *section, char *key, char *value) {
            return CONF_E_SUCCESS;
         return CONF_E_INTEXPECTED;
         break;
-        
+
     case CONF_T_MULTIPATH:
-        if(_conf_split(value,",",&valuearray) >= 0) {
+        if(util_split(value,",",&valuearray) >= 0) {
             index = 0;
             while(valuearray[index]) {
                 if(!_conf_existdir(valuearray[index])) {
-                    _conf_dispose_split(valuearray);
+                    util_dispose_split(valuearray);
                     return CONF_E_PATHEXPECTED;
                 }
                 index++;
             }
-            _conf_dispose_split(valuearray);
+            util_dispose_split(valuearray);
         }
         break;
 
@@ -534,7 +562,7 @@ void _conf_apply(LL_HANDLE pll) {
 
 /**
  * reload the existing config file.
- * 
+ *
  * @returns CONF_E_SUCCESS on success
  */
 int conf_reload(void) {
@@ -548,7 +576,6 @@ int conf_reload(void) {
  * @returns TRUE if successful, FALSE otherwise
  */
 int conf_read(char *file) {
-    FILE *fin;
     int err;
     LL_HANDLE pllnew, plltemp, pllcurrent, pllcomment;
     LL_ITEM *pli;
@@ -569,6 +596,11 @@ int conf_read(char *file) {
     char **valuearray;
     int index;
     char *temp;
+    IOHANDLE hconfig;
+    uint32_t len;
+
+    char *replaced_section = NULL;  /* config key/value updates */
+    char *replaced_key = NULL;      /* config key/value updates */
 
     if(conf_main_file) {
         conf_close();
@@ -578,10 +610,17 @@ int conf_read(char *file) {
     realpath(file,conf_file);
     conf_main_file = strdup(conf_file);
 
-    fin=fopen(conf_file,"r");
-    if(!fin) {
+    hconfig = io_new();
+    if(!hconfig)
+        DPRINTF(E_FATAL,L_CONF,"Malloc eror in io_new()\n");
+
+    DPRINTF(E_DBG,L_CONF,"Loading config file %s\n",conf_file);
+    if(!io_open(hconfig,"file://%U?ascii=1",conf_file)) {
+        DPRINTF(E_LOG,L_MISC,"Error opening config file: %s\n",io_errstr(hconfig));
+        io_dispose(hconfig);
         return CONF_E_FOPEN;
     }
+    DPRINTF(E_DBG,L_CONF,"Config file open\n");
 
     prev_comments = (char*)malloc(total_comment_length);
     if(!prev_comments)
@@ -590,19 +629,24 @@ int conf_read(char *file) {
 
     if((err=ll_create(&pllnew)) != LL_E_SUCCESS) {
         DPRINTF(E_LOG,L_CONF,"Error creating linked list: %d\n",err);
-        fclose(fin);
+        io_close(hconfig);
+        io_dispose(hconfig);
+
         return CONF_E_UNKNOWN;
     }
 
     ll_create(&pllcomment);  /* don't care if we lose comments */
 
     comment = NULL;
-    pllcurrent=NULL;
+    pllcurrent = NULL;
 
     /* got what will be the root of the config tree, now start walking through
      * the input file, populating the tree
      */
-    while(fgets(linebuffer,CONF_LINEBUFFER,fin)) {
+
+    /* TODO: linebuffered IO support */
+    len = sizeof(linebuffer);
+    while(io_readline(hconfig,(unsigned char *)linebuffer,&len) && len) {
         line++;
         linebuffer[CONF_LINEBUFFER] = '\0';
         ws=0;
@@ -633,21 +677,22 @@ int conf_read(char *file) {
 
             if(!value) {
                 ll_destroy(pllnew);
-                fclose(fin);
+                io_close(hconfig);
+                io_dispose(hconfig);
                 return CONF_E_BADHEADER;
             }
             *value = '\0';
 
             if((err = ll_create(&plltemp)) != LL_E_SUCCESS) {
                 ll_destroy(pllnew);
-                fclose(fin);
+                io_close(hconfig);
+                io_dispose(hconfig);
                 return CONF_E_UNKNOWN;
             }
 
             ll_add_ll(pllnew,term,plltemp);
 
             /* set current section and name */
-            pllcurrent = plltemp;
             if(section_name)
                 free(section_name);
             section_name = strdup(term);
@@ -686,11 +731,11 @@ int conf_read(char *file) {
             strsep(&value,delim);
             if((value) && (term) && (strlen(term))) {
                 while((strlen(term) && (strchr("\t ",term[strlen(term)-1]))))
-                      term[strlen(term)-1] = '\0';
+                    term[strlen(term)-1] = '\0';
                 while(strlen(value) && (strchr("\t ",*value)))
                     value++;
                 while((strlen(value) && (strchr("\t ",value[strlen(value)-1]))))
-                      value[strlen(value)-1] = '\0';
+                    value[strlen(value)-1] = '\0';
 
                 /* convert spaces to underscores in key */
                 temp = term;
@@ -700,16 +745,16 @@ int conf_read(char *file) {
                     temp++;
                 }
 
-                if(!pllcurrent) {
+                if(!section_name) {
                     /* in compat mode -- add a general section */
                     if((err=ll_create(&plltemp)) != LL_E_SUCCESS) {
                         DPRINTF(E_LOG,L_CONF,"Error creating list: %d\n",err);
                         ll_destroy(pllnew);
-                        fclose(fin);
+                        io_close(hconfig);
+                        io_dispose(hconfig);
                         return CONF_E_UNKNOWN;
                     }
                     ll_add_ll(pllnew,"general",plltemp);
-                    pllcurrent = plltemp;
 
                     if(section_name)
                         free(section_name); /* shouldn't ahppen */
@@ -726,7 +771,27 @@ int conf_read(char *file) {
                 }
 
                 /* see what kind this is, and act accordingly */
-                pce = _conf_get_keyinfo(section_name,term);
+                _conf_remap_entry(section_name, term, &replaced_section, &replaced_key);
+                DPRINTF(E_DBG,L_CONF,"Got %s/%s, convert to %s/%s (%s)\n",section_name,
+                        term, replaced_section, replaced_key, value);
+
+                pli = ll_fetch_item(pllnew,replaced_section);
+                if(pli) {
+                    DPRINTF(E_DBG,L_CONF,"Found existing section\n");
+                    pllcurrent = pli->value.as_ll;
+                } else {
+                    DPRINTF(E_DBG,L_CONF,"creating new section\n");
+                    if((err = ll_create(&pllcurrent)) != LL_E_SUCCESS) {
+                        ll_destroy(pllnew);
+                        DPRINTF(E_LOG,L_CONF,"Error creating linked list: %d\n",err);
+                        io_close(hconfig);
+                        io_dispose(hconfig);
+                        return CONF_E_UNKNOWN;
+                    }
+                    ll_add_ll(pllnew,replaced_section,pllcurrent);
+                }
+                pce = _conf_get_keyinfo(replaced_section, replaced_key);
+
                 key_type = CONF_T_STRING;
                 if(pce)
                     key_type = pce->type;
@@ -735,35 +800,35 @@ int conf_read(char *file) {
                 case CONF_T_MULTIPATH:
                 case CONF_T_MULTICOMMA:
                     /* first, see if we already have a tree... */
-                    pli = ll_fetch_item(pllcurrent,term);
+                    pli = ll_fetch_item(pllcurrent,replaced_key);
                     if(!pli) {
                         if((ll_create(&plltemp) != LL_E_SUCCESS)) {
                             DPRINTF(E_FATAL,L_CONF,"Could not create "
                                     "linked list.\n");
                         }
-                        ll_add_ll(pllcurrent,term,plltemp);
+                        ll_add_ll(pllcurrent,replaced_key,plltemp);
                         ll_set_flags(plltemp,0); /* allow dups */
                     } else {
                         plltemp = pli->value.as_ll;
                     }
 
                     /* got list, break comma sep and add */
-                    if(_conf_split(value,",",&valuearray) >= 0) {
+                    if(util_split(value,",",&valuearray) >= 0) {
                         index = 0;
                         while(valuearray[index]) {
-                            ll_add_string(plltemp,term,valuearray[index]);
+                            ll_add_string(plltemp,replaced_key,valuearray[index]);
                             index++;
                         }
-                        _conf_dispose_split(valuearray);
+                        util_dispose_split(valuearray);
                     } else {
-                        ll_add_string(plltemp,term,value);
+                        ll_add_string(plltemp,replaced_key,value);
                     }
                     break;
                 case CONF_T_INT:
                 case CONF_T_STRING:
                 case CONF_T_EXISTPATH:
                 default:
-                    ll_add_string(pllcurrent,term,value);
+                    ll_add_string(pllcurrent,replaced_key,value);
                     break;
                 }
 
@@ -771,7 +836,7 @@ int conf_read(char *file) {
                 if(comment) {
                     /* this is an inline comment */
                     snprintf(keybuffer,sizeof(keybuffer),"in_%s_%s",
-                             section_name,term);
+                             replaced_section,replaced_key);
                     ll_add_string(pllcomment,keybuffer,comment);
                     comment = NULL;
                 }
@@ -779,7 +844,7 @@ int conf_read(char *file) {
                 if(prev_comments[0] != '\0') {
                     /* we had some preceding comments */
                     snprintf(keybuffer,sizeof(keybuffer),"pre_%s_%s",
-                             section_name, term);
+                             replaced_section, replaced_key);
                     ll_add_string(pllcomment,keybuffer,prev_comments);
                     prev_comments[0] = '\0';
                     current_comment_length=0;
@@ -790,6 +855,7 @@ int conf_read(char *file) {
 
             if(((term) && (strlen(term))) && (!value)) {
                 DPRINTF(E_LOG,L_CONF,"Error in config file on line %d\n",line);
+                DPRINTF(E_LOG,L_CONF,"key: %s, value: %s\n",replaced_key,value);
                 ll_destroy(pllnew);
                 return CONF_E_PARSE;
             }
@@ -799,7 +865,6 @@ int conf_read(char *file) {
             if(!comment)
                 comment = "";
 
-
             /* add to prev comments */
             while((current_comment_length + (int)strlen(comment) + 2 >=
                    total_comment_length) && (total_comment_length < 32768)) {
@@ -827,24 +892,29 @@ int conf_read(char *file) {
                 }
             }
         }
+        len = sizeof(linebuffer);
     }
 
-    if(section_name)
+    if(section_name) {
         free(section_name);
+        section_name = NULL;
+    }
 
     if(prev_comments) {
         if(prev_comments[0] != '\0') {
             ll_add_string(pllcomment,"end",prev_comments);
         }
         free(prev_comments);
+        prev_comments = NULL;
     }
 
-    fclose(fin);
+    io_close(hconfig);
+    io_dispose(hconfig);
 
     /*  Sanity check */
     if(_conf_verify(pllnew)) {
         DPRINTF(E_INF,L_CONF,"Loading new config file.\n");
-        _conf_lock();
+        util_mutex_lock(l_conf);
         _conf_apply(pllnew);
         if(conf_main) {
             ll_destroy(conf_main);
@@ -856,7 +926,7 @@ int conf_read(char *file) {
 
         conf_main = pllnew;
         conf_comments = pllcomment;
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
     } else {
         ll_destroy(pllnew);
         ll_destroy(pllcomment);
@@ -902,14 +972,14 @@ int conf_get_int(char *section, char *key, int dflt) {
     LL_ITEM *pitem;
     int retval;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     pitem = _conf_fetch_item(conf_main,section,key);
     if((!pitem) || (pitem->type != LL_TYPE_STRING)) {
         retval = dflt;
     } else {
         retval = atoi(pitem->value.as_string);
     }
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
 
     return retval;
 }
@@ -930,7 +1000,7 @@ int conf_get_string(char *section, char *key, char *dflt, char *out, int *size)
     char *result;
     int len;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     pitem = _conf_fetch_item(conf_main,section,key);
     if((!pitem) || (pitem->type != LL_TYPE_STRING)) {
         result = dflt;
@@ -939,7 +1009,7 @@ int conf_get_string(char *section, char *key, char *dflt, char *out, int *size)
     }
 
     if(!result) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return CONF_E_NOTFOUND;
     }
 
@@ -949,12 +1019,12 @@ int conf_get_string(char *section, char *key, char *dflt, char *out, int *size)
         *size = len;
         strcpy(out,result);
     } else {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         *size = len;
         return CONF_E_OVERFLOW;
     }
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return CONF_E_SUCCESS;
 }
 
@@ -972,7 +1042,7 @@ char *conf_alloc_string(char *section, char *key, char *dflt) {
     char *result;
     char *retval;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     pitem = _conf_fetch_item(conf_main,section,key);
     if((!pitem) || (pitem->type != LL_TYPE_STRING)) {
         result = dflt;
@@ -981,7 +1051,7 @@ char *conf_alloc_string(char *section, char *key, char *dflt) {
     }
 
     if(result == NULL) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return NULL;
     }
 
@@ -990,7 +1060,7 @@ char *conf_alloc_string(char *section, char *key, char *dflt) {
     if(!retval) {
         DPRINTF(E_FATAL,L_CONF,"Malloc error in conf_alloc_string\n");
     }
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return retval;
 }
 
@@ -1033,17 +1103,18 @@ int conf_set_string(char *section, char *key, char *value, int verify) {
     char *oldvalue=NULL;
     LL_ITEM *polditem;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
 
     /* verify the item */
     err=_conf_verify_element(section,key,value);
     if(err != CONF_E_SUCCESS) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
+        DPRINTF(E_DBG,L_CONF,"Couldn't validate %s = %s\n",key,value);
         return err;
     }
 
     if(verify) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return CONF_E_SUCCESS;
     }
 
@@ -1064,19 +1135,19 @@ int conf_set_string(char *section, char *key, char *value, int verify) {
         /* deleting the item */
         pitem = ll_fetch_item(conf_main,section);
         if(!pitem) {
-            _conf_unlock();
+            util_mutex_unlock(l_conf);
             return CONF_E_SUCCESS;
         }
 
         section_ll = pitem->value.as_ll;
         if(!section_ll) {
-            _conf_unlock();
+            util_mutex_unlock(l_conf);
             return CONF_E_SUCCESS; /* ?? deleting an already deleted item */
         }
 
         /* don't care about item... might already be gone! */
         ll_del_item(section_ll,key);
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return CONF_E_SUCCESS;
     }
 
@@ -1087,12 +1158,12 @@ int conf_set_string(char *section, char *key, char *value, int verify) {
             /* that subkey doesn't exist yet... */
             if((err = ll_create(&section_ll)) != LL_E_SUCCESS) {
                 DPRINTF(E_LOG,L_CONF,"Could not create linked list: %d\n",err);
-                _conf_unlock();
+                util_mutex_unlock(l_conf);
                 return CONF_E_UNKNOWN;
             }
             if((err=ll_add_ll(conf_main,section,section_ll)) != LL_E_SUCCESS) {
                 DPRINTF(E_LOG,L_CONF,"Error inserting new subkey: %d\n",err);
-                _conf_unlock();
+                util_mutex_unlock(l_conf);
                 return CONF_E_UNKNOWN;
             }
         } else {
@@ -1105,19 +1176,19 @@ int conf_set_string(char *section, char *key, char *value, int verify) {
             }
             ll_add_ll(section_ll,key,temp_ll);
             ll_set_flags(temp_ll,0); /* allow dups */
-            if(_conf_split(value,",",&valuearray) >= 0) {
+            if(util_split(value,",",&valuearray) >= 0) {
                 index = 0;
                 while(valuearray[index]) {
                     ll_add_string(temp_ll,key,valuearray[index]);
                     index++;
                 }
-                _conf_dispose_split(valuearray);
+                util_dispose_split(valuearray);
             }
         } else {
             if((err = ll_add_string(section_ll,key,value)) != LL_E_SUCCESS) {
                 DPRINTF(E_LOG,L_CONF,"Error in conf_set_string: "
                         "(%s/%s)\n",section,key);
-                _conf_unlock();
+                util_mutex_unlock(l_conf);
                 return CONF_E_UNKNOWN;
             }
         }
@@ -1131,20 +1202,20 @@ int conf_set_string(char *section, char *key, char *value, int verify) {
                         "conf_set_string: could not create ll\n");
             }
             ll_set_flags(pitem->value.as_ll,0); /* allow dups */
-            if(_conf_split(value,",",&valuearray) >= 0) {
+            if(util_split(value,",",&valuearray) >= 0) {
                 index = 0;
                 while(valuearray[index]) {
                     ll_add_string(pitem->value.as_ll,key,valuearray[index]);
                     index++;
                 }
-                _conf_dispose_split(valuearray);
+                util_dispose_split(valuearray);
             }
         } else {
             ll_update_string(pitem,value);
         }
     }
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return conf_write();
 }
 
@@ -1157,14 +1228,14 @@ int conf_iswritable(void) {
     int retval = FALSE;
 
     /* don't want configfile reopened under us */
-    _conf_lock();
+    util_mutex_lock(l_conf);
 
     if(!conf_main_file)
         return FALSE;
 
     retval = !access(conf_main_file,W_OK);
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return retval;
 }
 
@@ -1174,20 +1245,30 @@ int conf_iswritable(void) {
  */
 int conf_write(void) {
     int retval = CONF_E_NOTWRITABLE;
-    FILE *fp;
+    IOHANDLE outfile;
 
     if(!conf_main_file) {
+        DPRINTF(E_DBG,L_CONF,"Config file apparently  not loaded\n");
         return CONF_E_NOCONF;
     }
 
-    _conf_lock();
-    if((fp = fopen(conf_main_file,"w+")) != NULL) {
-        retval = _conf_write(fp,conf_main,0,NULL);
-        fclose(fp);
-    }
-    _conf_unlock();
+    util_mutex_lock(l_conf);
+    outfile = io_new();
+    if(!outfile)
+        DPRINTF(E_FATAL,L_CONF,"io_new failed in conf_write\n");
 
-    return retval ? CONF_E_SUCCESS : CONF_E_NOTWRITABLE;
+    if(io_open(outfile,"file://%U?mode=w&ascii=1",conf_main_file)) {
+        retval = _conf_write(outfile,conf_main,0,NULL);
+        io_close(outfile);
+        io_dispose(outfile);
+    } else {
+        DPRINTF(E_LOG,L_CONF,"Error opening config file for write: %s\n",
+            io_errstr(outfile));
+        io_dispose(outfile);
+    }
+    util_mutex_unlock(l_conf);
+
+    return retval;
 }
 
 /**
@@ -1196,18 +1277,19 @@ int conf_write(void) {
  * @param fp file we are writing the config file to
  * @param pll list we are dumping k/v pairs for
  * @param sublevel whether this is the root, or a subkey
- * @returns TRUE on success, FALSE otherwise
+ * @returns CONF_E_SUCCESS on success, failure code otherwise
  */
-int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent) {
+int _conf_write(IOHANDLE hfile, LL *pll, int sublevel, char *parent) {
     LL_ITEM *pli;
     LL_ITEM *ppre, *pin;
     LL_ITEM *plitemp;
     int first;
+    int err;
 
     char keybuffer[256];
 
     if(!pll)
-        return TRUE;
+        return CONF_E_SUCCESS; /* ?? */
 
     /* write all the solo keys, first! */
     pli = pll->itemlist.next;
@@ -1226,7 +1308,7 @@ int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent) {
         }
 
         if(ppre) {
-            fprintf(fp,"%s",ppre->value.as_string);
+            io_printf(hfile,"%s",ppre->value.as_string);
         }
 
         switch(pli->type) {
@@ -1235,40 +1317,42 @@ int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent) {
                 /* must be multivalued */
                 plitemp = NULL;
                 first = 1;
-                fprintf(fp,"%s = ",pli->key);
+                io_printf(hfile,"%s = ",pli->key);
                 while((plitemp = ll_get_next(pli->value.as_ll,plitemp))) {
                     if(!first)
-                        fprintf(fp,",");
+                        io_printf(hfile,",");
                     first=0;
-                    fprintf(fp,"%s",plitemp->value.as_string);
+                    io_printf(hfile,"%s",plitemp->value.as_string);
                 }
-                fprintf(fp,"\n");
+                io_printf(hfile,"\n");
             } else {
-                fprintf(fp,"[%s]",pli->key);
+                io_printf(hfile,"[%s]",pli->key);
                 if(pin) {
-                    fprintf(fp," #%s",pin->value.as_string);
+                    io_printf(hfile," #%s",pin->value.as_string);
                 }
-                fprintf(fp,"\n");
+                io_printf(hfile,"\n");
 
-                if(!_conf_write(fp, pli->value.as_ll, 1, pli->key))
-                   return FALSE;
+                if((err =_conf_write(hfile, pli->value.as_ll, 1, pli->key)) != CONF_E_SUCCESS) {
+                    DPRINTF(E_DBG,L_CONF,"Error writing key %s:%s\n",pli->key);
+                    return err;
+                }
             }
             break;
 
         case LL_TYPE_INT:
-            fprintf(fp,"%s = %d",pli->key,pli->value.as_int);
+            io_printf(hfile,"%s = %d",pli->key,pli->value.as_int);
             if(pin) {
-                fprintf(fp," #%s",pin->value.as_string);
+                io_printf(hfile," #%s",pin->value.as_string);
             }
-            fprintf(fp,"\n");
+            io_printf(hfile,"\n");
             break;
 
         case LL_TYPE_STRING:
-            fprintf(fp,"%s = %s",pli->key,pli->value.as_string);
+            io_printf(hfile,"%s = %s",pli->key,pli->value.as_string);
             if(pin) {
-                fprintf(fp," #%s",pin->value.as_string);
+                io_printf(hfile," #%s",pin->value.as_string);
             }
-            fprintf(fp,"\n");
+            io_printf(hfile,"\n");
             break;
         }
 
@@ -1278,11 +1362,11 @@ int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent) {
     if(!sublevel) {
         pin = ll_fetch_item(conf_comments,"end");
         if(pin) {
-            fprintf(fp,"%s",pin->value.as_string);
+            io_printf(hfile,"%s",pin->value.as_string);
         }
     }
 
-    return TRUE;
+    return CONF_E_SUCCESS;
 }
 
 
@@ -1296,104 +1380,15 @@ int _conf_write(FILE *fp, LL *pll, int sublevel, char *parent) {
 int conf_isset(char *section, char *key) {
     int retval = FALSE;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     if(_conf_fetch_item(conf_main,section,key)) {
         retval = TRUE;
     }
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
 
     return retval;
 }
 
-/**
- * split a string on delimiter boundaries, filling
- * a string-pointer array.
- *
- * The user must free both the first element in the array,
- * and the array itself.
- *
- * @param s string to split
- * @param delimiters boundaries to split on
- * @param argvp an argv array to be filled
- * @returns number of tokens
- */
-int _conf_split(char *s, char *delimiters, char ***argvp) {
-    int i;
-    int numtokens;
-    const char *snew;
-    char *t;
-    char *tokptr;
-    char *tmp;
-    char *fix_src, *fix_dst;
-
-    if ((s == NULL) || (delimiters == NULL) || (argvp == NULL))
-        return -1;
-    *argvp = NULL;
-    snew = s + strspn(s, delimiters);
-    if ((t = malloc(strlen(snew) + 1)) == NULL)
-        return -1;
-
-    strcpy(t, snew);
-    numtokens = 1;
-    tokptr = NULL;
-    tmp = t;
-
-    tmp = s;
-    while(*tmp) {
-        if(strchr(delimiters,*tmp) && (*(tmp+1) == *tmp)) {
-            tmp += 2;
-        } else if(strchr(delimiters,*tmp)) {
-            numtokens++;
-            tmp++;
-        } else {
-            tmp++;
-        }
-    }
-
-    DPRINTF(E_DBG,L_CONF,"Found %d tokens in %s\n",numtokens,s);
-
-    if ((*argvp = malloc((numtokens + 1)*sizeof(char *))) == NULL) {
-        free(t);
-        return -1;
-    }
-
-    if (numtokens == 0)
-        free(t);
-    else {
-        tokptr = t;
-        tmp = t;
-        for (i = 0; i < numtokens; i++) {
-            while(*tmp) {
-                if(strchr(delimiters,*tmp) && (*(tmp+1) != *tmp))
-                    break;
-                if(strchr(delimiters,*tmp)) {
-                    tmp += 2; 
-                } else {
-                    tmp++;
-                }
-            }
-            *tmp = '\0';
-            tmp++;
-            (*argvp)[i] = tokptr;
-            
-            fix_src = fix_dst = tokptr;
-            while(*fix_src) {
-                if(strchr(delimiters,*fix_src) && (*(fix_src+1) == *fix_src)) {
-                    fix_src++;
-                }
-                *fix_dst++ = *fix_src++;
-            }
-            *fix_dst = '\0';
-
-            tokptr = tmp;
-            DPRINTF(E_DBG,L_CONF,"Token %d: %s\n",i+1,(*argvp)[i]);
-        }
-    }
-
-    *((*argvp) + numtokens) = NULL;
-    return numtokens;
-}
-
 /**
  * implode a multivalued term in a perl sense.
  *
@@ -1409,10 +1404,10 @@ char *conf_implode(char *section, char *key, char *delimiter) {
     int len;
     char *retval;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     pitem = _conf_fetch_item(conf_main,section,key);
     if((!pitem) || (pitem->type != LL_TYPE_LL)) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return NULL;
     }
 
@@ -1428,7 +1423,7 @@ char *conf_implode(char *section, char *key, char *delimiter) {
     }
 
     if(!count) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return NULL;
     }
 
@@ -1447,26 +1442,10 @@ char *conf_implode(char *section, char *key, char *delimiter) {
         }
     }
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return retval;
 }
 
-/**
- * dispose of the argv set that was created in _conf_split
- *
- * @param argv string array to delete
- */
-void _conf_dispose_split(char **argv) {
-    if(!argv)
-        return;
-
-    if(argv[0])
-        free(argv[0]);
-
-    free(argv);
-}
-
-
 /**
  * return a multi-valued item as an array (values)
  *
@@ -1479,10 +1458,10 @@ int conf_get_array(char *section, char *key, char ***argvp) {
     int count;
     int len;
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
     pitem = _conf_fetch_item(conf_main,section,key);
     if((!pitem) || (pitem->type != LL_TYPE_LL)) {
-        _conf_unlock();
+        util_mutex_unlock(l_conf);
         return FALSE;
     }
 
@@ -1515,7 +1494,7 @@ int conf_get_array(char *section, char *key, char ***argvp) {
         count++;
     }
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
     return TRUE;
 }
 
@@ -1530,7 +1509,6 @@ void conf_dispose_array(char **argv) {
     if(!argv)
         return;
 
-
     while(argv[index]) {
         free(argv[index]);
         index++;
@@ -1556,11 +1534,11 @@ int conf_xml_dump(WS_CONNINFO *pwsc) {
     pxml = xml_init(pwsc,1);
     xml_push(pxml,"config");
 
-    _conf_lock();
+    util_mutex_lock(l_conf);
 
     retval = _conf_xml_dump(pxml,conf_main,0,NULL);
 
-    _conf_unlock();
+    util_mutex_unlock(l_conf);
 
     xml_pop(pxml);
     xml_deinit(pxml);
@@ -1631,23 +1609,91 @@ char *conf_get_filename(void) {
 
 /**
  * this is an ugly block of crap to carry around every
- * time one wants the servername. 
+ * time one wants the servername.
  */
 char *conf_get_servername(void) {
     char *retval;
-    char newname[HOST_NAME_MAX + 1 + 16]; /* " Firefly Server" */
+    char *default_name = "firefly %v on %h";
+    char hostname[HOST_NAME_MAX + 1]; /* " Firefly Server" */
+    char *template, *src, *dst;
+    int len;
 
-    gethostname(newname,HOST_NAME_MAX);
-    retval = strchr(newname,'.');
+    gethostname(hostname,HOST_NAME_MAX);
+    retval = strchr(hostname,'.');
     if(retval) *retval = '\0';
 
-    retval = newname;
+    retval = hostname;
     while(*retval) {
         *retval = tolower(*retval);
         retval++;
     }
 
-    strcat(newname," Firefly Server");
+    template = conf_alloc_string("general","servername",default_name);
+    src = template;
+    len = 0;
+    while(*src) {
+        if(*src == '%') {
+            src++;
+            switch(*src) {
+            case 'h':
+                len += strlen(hostname);
+                src++;
+                break;
+            case 'v':
+                len += strlen(VERSION);
+                src++;
+                break;
+            default:
+                len += 2;
+                src++;
+                break;
+            }
+        } else {
+            len++;
+            src++;
+        }
+    }
 
-    return conf_alloc_string("general","servername",newname);
+    retval = (char*)malloc(len+1);
+    if(!retval) DPRINTF(E_FATAL,L_MISC,"malloc");
+
+    memset(retval,0,len+1);
+
+    dst = retval;
+    src = template;
+
+    while(*src) {
+        if(*src == '%') {
+            src++;
+            switch(*src) {
+            case 'h':
+                strcat(dst,hostname);
+                dst += strlen(hostname);
+                src ++;
+                break;
+            case 'v':
+                strcat(dst,VERSION);
+                dst += strlen(VERSION);
+                src++;
+                break;
+            default:
+                *dst++ = '%';
+                *dst++ = *src++;
+                break;
+            }
+        } else {
+            *dst++ = *src++;
+        }
+
+    }
+
+    if(strlen(retval) > MAX_REND_LEN) {
+        retval[MAX_REND_LEN] = '\0';
+        retval[MAX_REND_LEN-1] = '.';
+        retval[MAX_REND_LEN-2] = '.';
+        retval[MAX_REND_LEN-3] = '.';
+    }
+
+    free(template);
+    return retval;
 }
