@@ -1,5 +1,5 @@
 /*
- * $Id: configfile.h 364 2004-10-25 04:50:59Z rpedde $
+ * $Id: configfile.h 1538 2007-04-15 23:57:00Z rpedde $
  * Functions for reading and writing the config file
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -25,13 +25,18 @@
 #include "daapd.h"
 #include "webserver.h"
 
-extern int config_read(char *file);
-extern int config_write(WS_CONNINFO *pwsc);
-extern int config_auth(char *user, char *password);
+extern int config_auth(WS_CONNINFO *pwsc, char *user, char *password);
 extern void config_handler(WS_CONNINFO *pwsc);
 extern void config_set_status(WS_CONNINFO *pwsc, int session, char *fmt, ...);
 extern int config_get_session_count(void);
-extern int config_get_next_session(void);
-extern void config_close(void);
+extern int config_matches_role(WS_CONNINFO *pwsc, char *username, char *password, char *role);
+
+/** thread local storage */
+typedef struct tag_scan_status {
+    int session;
+    int thread;
+    char *what;
+    char *host;
+} SCAN_STATUS;
 
 #endif /* _CONFIGFILE_H_ */
