@@ -1,5 +1,5 @@
 /*
- * $Id: daapd.h 1431 2006-11-15 23:11:16Z rpedde $
+ * $Id: daapd.h 1484 2007-01-17 01:06:16Z rpedde $
  * Header info for daapd server
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -31,13 +31,26 @@
 #ifndef _DAAPD_H_
 #define _DAAPD_H_
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
+
+#include <time.h>
+#include "memdebug.h"  /* redefine free/malloc/etc */
+#include "compat.h"
+#include "webserver.h"
+
 /** Simple struct for holding stat info.
  * \todo wire up the tag_stats#bytes_served stuff into r_write() in restart.c
  */
 typedef struct tag_stats {
     time_t start_time;          /**< When the server was started */
     int songs_served;           /**< How many songs have been served */
- 
+
     unsigned int gb_served;     /**< How many gigs of data have been served (unused) */
     unsigned int bytes_served;  /**< How many bytes of data served (unused) */
 } STATS;
@@ -47,32 +60,13 @@ typedef struct tag_config {
     int use_mdns;         /**< Should we do rendezvous advertisements? */
     int stop;             /**< Time to exit? */
     int reload;           /**< Time to reload and/or rescan the database? */
-    char *configfile;     /**< path to config file */
-    char *iface;          /**< interface to advertise on */
-    char *web_root;       /**< path to the directory containing the admin-root files */
-    int port;             /**< port to listen on */
-    int rescan_interval;  /**< How often to do a background rescan of the file system */
-    int always_scan;      /**< 0 to minimize disk usage (embedded devices) */
-    int process_m3u;      /**< Should we process m3u files? */
-    int scan_type;        /**< How hard to search mp3 files. see scan_get_mp3fileinfo() */
-    int compress;         /**< Should we compress? */
-    int pid;              /**< pid that will accept INT to terminate */
-    char *adminpassword;  /**< Password to web management pages */
-    char *readpassword;   /**< iTunes password */
-    char *mp3dir;         /**< root directory of the mp3 files */
-    char *servername;     /**< Name advertised via rendezvous */
-    char *playlist;       /**< Path to the playlist file */
-    char *runas;          /**< Who to drop privs to (if run as root) */
-    char *dbdir;          /**< Where to put the db file */
-    char *extensions;     /**< What music file extentions to process */
-    char *artfilename;    /**< What filename to merge coverart with */
-    char *logfile;        /**< What file to use as a logfile */
+    int foreground;       /**< Whether or not we are running in foreground */
+    int full_reload;      /**< Whether the reload should be a full one */
+
     STATS stats;          /**< Stats structure (see above) */
+    WSHANDLE server;      /**< webserver handle */
 } CONFIG;
 
 extern CONFIG config;
 
-/* Forwards */
-extern int drop_privs(char *user);
-
 #endif /* _DAAPD_H_ */
