@@ -5,6 +5,7 @@
  */
 
 #include <stdio.h>
+#include "daapd.h"
 
 char *db_sqlite_updates[] = {
     /* version 0 -> version 1 -- initial update */
@@ -47,7 +48,7 @@ char *db_sqlite_updates[] = {
     "   sample_count    INTEGER DEFAULT 0,\n"
     "   force_update    INTEGER DEFAULT 0\n"
     ");\n"
-    "CREATE INDEX idx_path ON songs(path);\n" 
+    "CREATE INDEX idx_path ON songs(path);\n"
     "CREATE TABLE config (\n"
     "   term            VARCHAR(255)    NOT NULL,\n"
     "   subterm         VARCHAR(255)    DEFAULT NULL,\n"
@@ -141,7 +142,7 @@ char *db_sqlite_updates[] = {
     "create index idx_path on songs(path);\n"
     "drop table tempsongs;\n"
     "update config set value=3 where term='version';\n",
-    
+
     /* version 3 -> version 4 */
     /* add db_timestamp and path to playlist table */
     "create temp table tempplaylists as select * from playlists;\n"
@@ -240,13 +241,13 @@ char *db_sqlite_updates[] = {
     "insert into playlistitems (playlistid, songid) select * from tempitems;\n"
     "drop table tempitems;\n"
     "update config set value=7 where term='version';\n",
-    
+
     /* version 7 -> version 8 */
     "create index idx_songid on playlistitems(songid)\n"
     "create index idx_playlistid on playlistitems(playlistid)\n"
     "update config set value=8 where term='version';\n",
 
-    /* version 8 -> version 9 */ 
+    /* version 8 -> version 9 */
     "drop index idx_path;\n"
     "create temp table tempsongs as select * from songs;\n"
     "drop table songs;\n"
@@ -301,7 +302,7 @@ char *db_sqlite_updates[] = {
     "drop table tempsongs;\n"
     "update config set value=9 where term='version';\n",
 
-    /* version 9 -> version 10 */ 
+    /* version 9 -> version 10 */
     "drop index idx_path;\n"
     "create temp table tempsongs as select * from songs;\n"
     "drop table songs;\n"
@@ -354,12 +355,66 @@ char *db_sqlite_updates[] = {
     "create index idx_path on songs(path,idx);\n"
     "drop table tempsongs;\n"
     "update config set value=10 where term='version';\n",
-    /* version 10 -> version 11 */ 
+    /* version 10 -> version 11 */
     "drop index idx_playlistid;\n"
     "create index idx_playlistid on playlistitems(playlistid,songid);\n"
     "update config set value=11 where term='version';\n",
     /* version 11 -> version 12 */
     "REPLACE INTO config VALUES('rescan',NULL,1);\n"
     "UPDATE config SET value=12 WHERE term='version';\n",
+    /* version 12 -> version 13 */
+    "create temp table tempsongs as select * from songs;\n"
+    "drop table songs;\n"
+    "CREATE TABLE songs (\n"
+    "   id              INTEGER PRIMARY KEY NOT NULL,\n"
+    "   path            VARCHAR(4096) NOT NULL,\n"
+    "   fname           VARCHAR(255) NOT NULL,\n"
+    "   title           VARCHAR(1024) DEFAULT NULL,\n"
+    "   artist          VARCHAR(1024) DEFAULT NULL,\n"
+    "   album           VARCHAR(1024) DEFAULT NULL,\n"
+    "   genre           VARCHAR(255) DEFAULT NULL,\n"
+    "   comment         VARCHAR(4096) DEFAULT NULL,\n"
+    "   type            VARCHAR(255) DEFAULT NULL,\n"
+    "   composer        VARCHAR(1024) DEFAULT NULL,\n"
+    "   orchestra       VARCHAR(1024) DEFAULT NULL,\n"
+    "   conductor       VARCHAR(1024) DEFAULT NULL,\n"
+    "   grouping        VARCHAR(1024) DEFAULT NULL,\n"
+    "   url             VARCHAR(1024) DEFAULT NULL,\n"
+    "   bitrate         INTEGER DEFAULT 0,\n"
+    "   samplerate      INTEGER DEFAULT 0,\n"
+    "   song_length     INTEGER DEFAULT 0,\n"
+    "   file_size       INTEGER DEFAULT 0,\n"
+    "   year            INTEGER DEFAULT 0,\n"
+    "   track           INTEGER DEFAULT 0,\n"
+    "   total_tracks    INTEGER DEFAULT 0,\n"
+    "   disc            INTEGER DEFAULT 0,\n"
+    "   total_discs     INTEGER DEFAULT 0,\n"
+    "   bpm             INTEGER DEFAULT 0,\n"
+    "   compilation     INTEGER DEFAULT 0,\n"
+    "   rating          INTEGER DEFAULT 0,\n"
+    "   play_count      INTEGER DEFAULT 0,\n"
+    "   data_kind       INTEGER DEFAULT 0,\n"
+    "   item_kind       INTEGER DEFAULT 0,\n"
+    "   description     INTEGER DEFAULT 0,\n"
+    "   time_added      INTEGER DEFAULT 0,\n"
+    "   time_modified   INTEGER DEFAULT 0,\n"
+    "   time_played     INTEGER DEFAULT 0,\n"
+    "   db_timestamp    INTEGER DEFAULT 0,\n"
+    "   disabled        INTEGER DEFAULT 0,\n"
+    "   sample_count    INTEGER DEFAULT 0,\n"
+    "   force_update    INTEGER DEFAULT 0,\n"
+    "   codectype       VARCHAR(5) DEFAULT NULL,\n"
+    "   idx             INTEGER NOT NULL,\n"
+    "   has_video       INTEGER DEFAULT 0,\n"
+    "   contentrating   INTEGER DEFAULT 0,\n"
+    "   bits_per_sample INTEGER DEFAULT 0,\n"
+    "   album_artist    VARCHAR(1024)\n"
+    ");\n"
+    "begin transaction;\n"
+    "insert into songs select *,0,'' from tempsongs;\n"
+    "commit transaction;\n"
+    "drop table tempsongs;\n"
+    "update config set value=13 where term='version';\n",
     NULL /* No more versions! */
+
 };
