@@ -1,5 +1,5 @@
 /*
- * $Id: db-sql.c 1374 2006-08-31 02:43:13Z rpedde $
+ * $Id: db-sql.c 1695 2007-10-24 05:44:35Z rpedde $
  * sql-specific db implementation
  *
  * Copyright (C) 2005 Ron Pedde (ron@pedde.com)
@@ -34,7 +34,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 
+#include "daapd.h"
 #include "err.h"
 #include "mp3-scanner.h"
 #include "db-generic.h"
@@ -60,8 +64,8 @@ static int db_sql_need_dispose=0;
 extern char *db_sqlite_updates[];
 
 /* Forwards */
-int db_sql_get_size(DBQUERYINFO *pinfo, char **valarray);
-int db_sql_build_dmap(DBQUERYINFO *pinfo, char **valarray, unsigned char *presult, int len);
+
+
 void db_sql_build_mp3file(char **valarray, MP3FILE *pmp3);
 int db_sql_update(char **pe, MP3FILE *pmp3, int *id);
 int db_sql_update_playlists(char **pe);
@@ -71,10 +75,11 @@ char *_db_proper_path(char *path);
 #define STR(a) (a) ? (a) : ""
 #define ISSTR(a) ((a) && strlen((a)))
 #define MAYBEFREE(a) { if((a)) free((a)); };
+/*
 #define DMAPLEN(a) (((a) && strlen(a)) ? (8+(int)strlen((a))) : \
                                          ((pinfo->zero_length) ? 8 : 0))
 #define EMIT(a) (pinfo->zero_length ? 1 : ((a) && strlen((a))) ? 1 : 0)
-
+*/
 /**
  * functions for the specific db backend
  */
@@ -149,6 +154,16 @@ char *_db_proper_path(char *path) {
 }
 
 
+int db_sql_atoi(const char *what) {
+    return what ? atoi(what) : 0;
+}
+char *db_sql_strdup(const char *what) {
+    return what ? (strlen(what) ? strdup(what) : NULL) : NULL;
+}
+
+uint64_t db_sql_atol(const char *what) {
+    return what ? atoll(what) : 0;
+}
 /**
  * escape a sql string, returning it the supplied buffer.
  * note that this uses the sqlite escape format -- use %q for quoted
@@ -342,7 +357,7 @@ int db_sql_open(char **pe, char *parameters) {
             return result;
 
         max_version = 0;
-        while(db_updates[max_version]) 
+        while(db_updates[max_version])
             max_version++;
 
         DPRINTF(E_DBG,L_DB,"Current db version: %d\n",current_version);
@@ -353,7 +368,7 @@ int db_sql_open(char **pe, char *parameters) {
                     current_version + 1);
             result = db_sql_exec_fn(pe,E_LOG,"%s",db_updates[current_version]);
             if(result != DB_E_SUCCESS) {
-                DPRINTF(E_LOG,L_DB,"Error upgrading db: %s\n", pe ? *pe : 
+                DPRINTF(E_LOG,L_DB,"Error upgrading db: %s\n", pe ? *pe :
                         "Unknown");
                 return result;
             }
@@ -371,10 +386,11 @@ int db_sql_open(char **pe, char *parameters) {
  *        on return, reload is set to 1 if the db MUST be rescanned
  * @returns DB_E_SUCCESS on success, error code otherwise
  */
-int db_sql_init(int *reload) {
+int db_sql_init(int reload) {
     int items;
     int rescan = 0;
     int err;
+    int do_reload = reload;
 
     err=db_sql_get_count(NULL,&items, countSongs);
     if(err != DB_E_SUCCESS)
@@ -382,23 +398,22 @@ int db_sql_init(int *reload) {
 
     /* check if a request has been written into the db (by a db upgrade?) */
     if(db_sql_fetch_int(NULL,&rescan,"select value from config where "
-                        "term='rescan'") == DB_E_SUCCESS)
-    {
+                        "term='rescan'") == DB_E_SUCCESS) {
         if(rescan)
-            *reload=1;
+            do_reload=1;
     }
 
 
-
-    if(*reload || (!items)) {
+    /* we could pass back a status to describe whether a reaload was
+     * required (for reasons other than expicit request)
+     */
+    if(do_reload || (!items)) {
         DPRINTF(E_LOG,L_DB,"Full reload...\n");
         db_sql_event_fn(DB_SQL_EVENT_FULLRELOAD);
         db_sql_reload=1;
-        *reload=1;
     } else {
         db_sql_event_fn(DB_SQL_EVENT_STARTUP);
         db_sql_reload=0;
-        *reload=0;
     }
 
     return DB_E_SUCCESS;
@@ -423,7 +438,7 @@ int db_sql_force_rescan(char **pe) {
         return result;
 
     return db_sql_exec_fn(pe,E_LOG,"update songs set force_update=1");
-    
+
 }
 
 /**
@@ -617,17 +632,24 @@ int db_sql_edit_playlist(char **pe, int id, char *name, char *clause) {
             return DB_E_DUPLICATE_PLAYLIST;
         }
 
-        if((playlist_type == PL_SMART)&&(clause))
-            return db_sql_exec_fn(pe,E_LOG,"update playlists set title='%q', "
+        if((playlist_type == PL_SMART)&&(clause)) {
+            result=db_sql_exec_fn(pe,E_LOG,"update playlists set title='%q', "
                                   "query='%q' where id=%d",name,clause,id);
 
-        return db_sql_exec_fn(pe,E_LOG,"update playlists set title='%q' "
-                              "where id=%d",name,id);
+        } else {
+            result=db_sql_exec_fn(pe,E_LOG,"update playlists set title='%q' "
+                                  "where id=%d",name,id);
+        }
+        db_sql_update_playlists(NULL);
+        return result;
     }
 
-    if((playlist_type == PL_SMART) && (clause))
-        return db_sql_exec_fn(pe,E_LOG,"update playlists set query='%q' "
-                              "where id=%d",clause,id);
+    if((playlist_type == PL_SMART) && (clause)) {
+        result= db_sql_exec_fn(pe,E_LOG,"update playlists set query='%q' "
+                               "where id=%d",clause,id);
+        db_sql_update_playlists(NULL);
+        return result;
+    }
 
     return DB_E_SUCCESS;  /* ?? */
 }
@@ -646,6 +668,7 @@ int db_sql_add_playlist(char **pe, char *name, int type, char *clause, char *pat
     char *criteria;
     char *estring;
     char *correct_path;
+    SQL_ROW row;
 
     result=db_sql_fetch_int(pe,&cnt,"select count(*) from playlists where "
                             "upper(title)=upper('%q')",name);
@@ -655,6 +678,12 @@ int db_sql_add_playlist(char **pe, char *name, int type, char *clause, char *pat
     }
 
     if(cnt != 0) { /* duplicate */
+        db_sql_fetch_row(NULL,&row, "select * from playlists where "
+                         "upper(title)=upper('%q')",name);
+        DPRINTF(E_LOG,L_MISC,"Attempt to add duplicate playlist: '%s' "
+                "type: %d, path: %s, idx: %d\n",name,atoi(row[PL_TYPE]),
+                row[PL_PATH],atoi(row[PL_IDX]));
+        db_sql_dispose_row();
         db_get_error(pe,DB_E_DUPLICATE_PLAYLIST,name);
         return DB_E_DUPLICATE_PLAYLIST;
     }
@@ -666,16 +695,21 @@ int db_sql_add_playlist(char **pe, char *name, int type, char *clause, char *pat
 
     /* Let's throw it in  */
     switch(type) {
-    case PL_STATICWEB: /* static, maintained in web interface */
     case PL_STATICFILE: /* static, from file */
     case PL_STATICXML: /* from iTunes XML file */
         correct_path = _db_proper_path(path);
         result = db_sql_exec_fn(pe,E_LOG,"insert into playlists "
                                 "(title,type,items,query,db_timestamp,path,idx) "
                                  "values ('%q',%d,0,NULL,%d,'%q',%d)",
-                                 name,type,time(NULL),correct_path,index);
+                                 name,type,(int)time(NULL),correct_path,index);
         free(correct_path);
         break;
+    case PL_STATICWEB: /* static, maintained in web interface */
+        result = db_sql_exec_fn(pe,E_LOG,"insert into playlists "
+                                "(title,type,items,query,db_timestamp,path,idx) "
+                                 "values ('%q',%d,0,NULL,%d,NULL,%d)",
+                                 name,type,time(NULL),index);
+        break;
     case PL_SMART: /* smart */
         if(!db_sql_parse_smart(&estring,&criteria,clause)) {
             db_get_error(pe,DB_E_PARSE,estring);
@@ -703,6 +737,7 @@ int db_sql_add_playlist(char **pe, char *name, int type, char *clause, char *pat
         db_sql_exec_fn(NULL,E_FATAL,"insert into plupdated values (%d)",*playlistid);
     }
 
+    db_sql_update_playlists(NULL);
     return result;
 }
 
@@ -769,6 +804,8 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
     int insertid;
     char *query;
     char *path;
+    char sample_count[40];
+    char file_size[40];
 
     DPRINTF(E_SPAM,L_DB,"Entering db_sql_add\n");
 
@@ -801,6 +838,10 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
     pmp3->play_count=0;
     pmp3->time_played=0;
 
+    /* sqlite2 doesn't support 64 bit ints */
+    sprintf(sample_count,"%lld",pmp3->sample_count);
+    sprintf(file_size,"%lld",pmp3->file_size);
+
     err=db_sql_exec_fn(pe,E_DBG,"INSERT INTO songs VALUES "
                        "(NULL," // id
                        "'%q',"  // path
@@ -819,7 +860,7 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
                        "%d,"    // bitrate
                        "%d,"    // samplerate
                        "%d,"    // song_length
-                       "%d,"    // file_size
+                       "%s,"    // file_size
                        "%d,"    // year
                        "%d,"    // track
                        "%d,"    // total_tracks
@@ -837,12 +878,14 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
                        "%d,"    // time_played
                        "%d,"    // db_timestamp
                        "%d,"    // disabled
-                       "%d,"    // sample_count
+                       "%s,"    // sample_count
                        "0,"     // force_update
                        "'%q',"  // codectype
                        "%d,"    // index
                        "%d,"    // has_video
-                       "%d)",   // contentrating
+                       "%d,"    // contentrating
+                       "%d,"    // bits_per_sample
+                       "'%q')",   // albumartist
                        path,
                        STR(pmp3->fname),
                        STR(pmp3->title),
@@ -859,7 +902,7 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
                        pmp3->bitrate,
                        pmp3->samplerate,
                        pmp3->song_length,
-                       pmp3->file_size,
+                       file_size,
                        pmp3->year,
                        pmp3->track,
                        pmp3->total_tracks,
@@ -876,11 +919,13 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
                        pmp3->time_played,
                        pmp3->db_timestamp,
                        pmp3->disabled,
-                       pmp3->sample_count,
+                       sample_count,
                        STR(pmp3->codectype),
                        pmp3->index,
                        pmp3->has_video,
-                       pmp3->contentrating);
+                       pmp3->contentrating,
+                       pmp3->bits_per_sample,
+                       STR(pmp3->album_artist));
 
     free(path);
     if(err != DB_E_SUCCESS)
@@ -888,7 +933,8 @@ int db_sql_add(char **pe, MP3FILE *pmp3, int *id) {
 
     insertid = db_sql_insert_id_fn();
     if((db_sql_in_scan || db_sql_in_playlist_scan)&&(!db_sql_reload)) {
-        db_sql_exec_fn(NULL,E_FATAL,"insert into updated values (%d)",
+        /* fetch threads have different sqlite handle -- won't see the update table */
+        db_sql_exec_fn(NULL,E_DBG,"insert into updated values (%d)",
                        insertid);
     }
 
@@ -911,12 +957,17 @@ int db_sql_update(char **pe, MP3FILE *pmp3, int *id) {
     int err;
     char query[1024];
     char *path;
+    char sample_count[40];
+    char file_size[40];
 
     if(!pmp3->time_modified)
         pmp3->time_modified = (int)time(NULL);
 
     pmp3->db_timestamp = (int)time(NULL);
 
+    sprintf(sample_count,"%lld",pmp3->sample_count);
+    sprintf(file_size,"%lld",pmp3->file_size);
+
     strcpy(query,"UPDATE songs SET "
            "title='%q',"  // title
            "artist='%q',"  // artist
@@ -932,7 +983,7 @@ int db_sql_update(char **pe, MP3FILE *pmp3, int *id) {
            "bitrate=%d,"    // bitrate
            "samplerate=%d,"    // samplerate
            "song_length=%d,"    // song_length
-           "file_size=%d,"    // file_size
+           "file_size=%s,"    // file_size
            "year=%d,"    // year
            "track=%d,"    // track
            "total_tracks=%d,"    // total_tracks
@@ -944,8 +995,9 @@ int db_sql_update(char **pe, MP3FILE *pmp3, int *id) {
            "disabled=%d," // disabled
            "compilation=%d,"    // compilation
            "rating=%d,"    // rating
-           "sample_count=%d," // sample_count
-           "codectype='%q'"   // codec
+           "sample_count=%s," // sample_count
+           "codectype='%q',"   // codec
+           "album_artist='%q'"
            " WHERE path='%q' and idx=%d");
 
     path = _db_proper_path(pmp3->path);
@@ -965,7 +1017,7 @@ int db_sql_update(char **pe, MP3FILE *pmp3, int *id) {
                          pmp3->bitrate,
                          pmp3->samplerate,
                          pmp3->song_length,
-                         pmp3->file_size,
+                         file_size,
                          pmp3->year,
                          pmp3->track,
                          pmp3->total_tracks,
@@ -977,8 +1029,9 @@ int db_sql_update(char **pe, MP3FILE *pmp3, int *id) {
                          pmp3->disabled,
                          pmp3->compilation,
                          pmp3->rating,
-                         pmp3->sample_count,
+                         sample_count,
                          STR(pmp3->codectype),
+                         STR(pmp3->album_artist),
                          path,
                          pmp3->index);
 
@@ -1030,6 +1083,8 @@ int db_sql_update_playlists(char **pe) {
     SQL_ROW row;
     char *where_clause;
 
+    DPRINTF(E_LOG,L_DB,"Updating playlists\n");
+
     /* FIXME: There is a race here for externally added playlists */
 
     err = db_sql_fetch_int(pe,&playlists,"select count(*) from playlists");
@@ -1054,9 +1109,9 @@ int db_sql_update_playlists(char **pe) {
           (index < playlists))
     {
         /* process row */
-        pinfo[index].plid=strdup(STR(row[0]));
-        pinfo[index].type=strdup(STR(row[1]));
-        pinfo[index].clause=strdup(STR(row[2]));
+        pinfo[index].plid=strdup(STR(row[PL_ID]));
+        pinfo[index].type=strdup(STR(row[PL_TYPE]));
+        pinfo[index].clause=strdup(STR(row[PL_QUERY]));
         DPRINTF(E_SPAM,L_DB,"Found playlist %s: type %s, clause %s\n",pinfo[index].plid,
             pinfo[index].type,pinfo[index].clause);
         index++;
@@ -1270,7 +1325,7 @@ int db_sql_enum_start(char **pe, DBQUERYINFO *pinfo) {
             strcpy(query_rest," where (");
             have_clause = 1;
         }
-        
+
         switch(pinfo->query_type) {
         case queryTypeBrowseAlbums:
             strcat(query_rest,"album");
@@ -1299,7 +1354,7 @@ int db_sql_enum_start(char **pe, DBQUERYINFO *pinfo) {
         strcat(scratch,query_rest);
         if(browse)
             strcat(scratch,")");
-        
+
         err = db_sql_fetch_int(pe,&results,"%s",scratch);
         if(err != DB_E_SUCCESS)
             return err;
@@ -1344,40 +1399,6 @@ int db_sql_enum_start(char **pe, DBQUERYINFO *pinfo) {
     return err;
 }
 
-/**
- * find the size of the response by walking through the query and
- * sizing it
- *
- * @returns DB_E_SUCCESS on success, error code otherwise
- */
-int db_sql_enum_size(char **pe, DBQUERYINFO *pinfo, int *count, int *total_size) {
-    int err;
-    int record_size;
-    SQL_ROW row;
-
-    DPRINTF(E_DBG,L_DB,"Enumerating size\n");
-
-    *count=0;
-    *total_size = 0;
-
-    while(((err=db_sql_enum_fetch_fn(pe,&row)) == DB_E_SUCCESS) && (row)) {
-        if((record_size = db_sql_get_size(pinfo,row))) {
-            *total_size += record_size;
-            *count = *count + 1;
-        }
-    }
-
-    if(err != DB_E_SUCCESS) {
-        db_sql_enum_end_fn(NULL);
-        return err;
-    }
-
-    err=db_sql_enum_restart_fn(pe);
-
-    DPRINTF(E_DBG,L_DB,"Got size: %d\n",*total_size);
-    return err;
-}
-
 
 /**
  * fetch the next row in raw row format
@@ -1394,40 +1415,6 @@ int db_sql_enum_fetch_row(char **pe, PACKED_MP3FILE *row, DBQUERYINFO *pinfo) {
     return DB_E_SUCCESS;
 }
 
-/**
- * fetch the next record from the enum
- */
-int db_sql_enum_fetch(char **pe, DBQUERYINFO *pinfo, int *size, unsigned char **pdmap) {
-    int err;
-    int result_size=0;
-    unsigned char *presult;
-    SQL_ROW row;
-
-    err=db_sql_enum_fetch_fn(pe, &row);
-    if(err != DB_E_SUCCESS) {
-        db_sql_enum_end_fn(NULL);
-        return err;
-    }
-
-    if(row) {
-        result_size = db_sql_get_size(pinfo,row);
-        if(result_size) {
-            presult = (unsigned char*)malloc(result_size);
-            if(!presult) {
-                DPRINTF(E_FATAL,L_DB,"Malloc error\n");
-            }
-
-            db_sql_build_dmap(pinfo,row,presult,result_size);
-            *pdmap=presult;
-            *size = result_size;
-        }
-    } else {
-        *size = 0;
-    }
-
-    return DB_E_SUCCESS;
-}
-
 /**
  * start the enum again
  */
@@ -1443,329 +1430,6 @@ int db_sql_enum_end(char **pe) {
     return db_sql_enum_end_fn(pe);
 }
 
-/**
- * get the size of the generated dmap, given a specific meta
- */
-int db_sql_get_size(DBQUERYINFO *pinfo, SQL_ROW valarray) {
-    int size;
-    int transcode;
-
-    switch(pinfo->query_type) {
-    case queryTypeBrowseArtists: /* simple 'mlit' entry */
-    case queryTypeBrowseAlbums:
-    case queryTypeBrowseGenres:
-    case queryTypeBrowseComposers:
-        return valarray[0] ? (8 + (int) strlen(valarray[0])) : 0;
-    case queryTypePlaylists:
-        size = 8;   /* mlit */
-        size += 12; /* mimc - you get it whether you want it or not */
-        if(db_wantsmeta(pinfo->meta, metaItemId))
-            size += 12; /* miid */
-        if(db_wantsmeta(pinfo->meta, metaItunesSmartPlaylist)) {
-            if(valarray[plType] && (atoi(valarray[plType])==1))
-                size += 9;  /* aeSP */
-        }
-        if(db_wantsmeta(pinfo->meta, metaItemName))
-            size += (8 + (int) strlen(valarray[plTitle])); /* minm */
-        if(valarray[plType] && (atoi(valarray[plType])==1) &&
-           db_wantsmeta(pinfo->meta, metaMPlaylistSpec))
-            size += (8 + (int) strlen(valarray[plQuery])); /* MSPS */
-        if(db_wantsmeta(pinfo->meta, metaMPlaylistType))
-            size += 9; /* MPTY */
-        return size;
-        break;
-    case queryTypeItems:
-    case queryTypePlaylistItems:  /* essentially the same query */
-        /* see if this is going to be transcoded */
-        transcode = plugin_ssc_should_transcode(pinfo->pwsc,valarray[37]);
-
-        /* Items that get changed by transcode:
-         *
-         * type:         item  8: changes to 'wav'
-         * description:  item 29: changes to 'wav audio file'
-         * bitrate:      item 15: guestimated, based on item 15, samplerate
-         *
-         * probably file size should change as well, but currently doesn't
-         */
-
-        size = 8; /* mlit */
-        if(db_wantsmeta(pinfo->meta, metaItemKind))
-            /* mikd */
-            size += 9;
-        if(db_wantsmeta(pinfo->meta, metaSongDataKind))
-            /* asdk */
-            size += 9;
-        if(db_wantsmeta(pinfo->meta, metaSongDataURL))
-            /* asul */
-            size += DMAPLEN(valarray[13]);
-        if(db_wantsmeta(pinfo->meta, metaSongAlbum))
-            /* asal */
-            size += DMAPLEN(valarray[5]);
-        if(db_wantsmeta(pinfo->meta, metaSongArtist))
-            /* asar */
-            size += DMAPLEN(valarray[4]);
-        if(valarray[23] && atoi(valarray[23]) && db_wantsmeta(pinfo->meta, metaSongBPM))
-            /* asbt */
-            size += 10;
-        if(db_wantsmeta(pinfo->meta, metaSongBitRate)) {
-            /* asbr */
-            if(transcode) {
-                if(valarray[15] && atoi(valarray[15]))
-                    size += 10;
-            } else {
-                if(valarray[14] && atoi(valarray[14]))
-                    size += 10;
-            }
-        }
-        if(db_wantsmeta(pinfo->meta, metaSongComment))
-            /* ascm */
-            size += DMAPLEN(valarray[7]);
-        if(valarray[24] && atoi(valarray[24]) && db_wantsmeta(pinfo->meta,metaSongCompilation))
-            /* asco */
-            size += 9;
-        if(db_wantsmeta(pinfo->meta, metaSongComposer))
-            /* ascp */
-            size += DMAPLEN(valarray[9]);
-        if(db_wantsmeta(pinfo->meta, metaSongGrouping))
-            /* agrp */
-            size += DMAPLEN(valarray[12]);
-        if(valarray[30] && atoi(valarray[30]) && db_wantsmeta(pinfo->meta, metaSongDateAdded))
-            /* asda */
-            size += 12;
-        if(valarray[31] && atoi(valarray[31]) && db_wantsmeta(pinfo->meta,metaSongDateModified))
-            /* asdm */
-            size += 12;
-        if(valarray[22] && atoi(valarray[22]) && db_wantsmeta(pinfo->meta, metaSongDiscCount))
-            /* asdc */
-            size += 10;
-        if(valarray[21] && atoi(valarray[21]) && db_wantsmeta(pinfo->meta, metaSongDiscNumber))
-            /* asdn */
-            size += 10;
-        if(db_wantsmeta(pinfo->meta, metaSongGenre))
-            /* asgn */
-            size += DMAPLEN(valarray[6]);
-        if(db_wantsmeta(pinfo->meta,metaItemId))
-            /* miid */
-            size += 12;
-        if(db_wantsmeta(pinfo->meta,metaSongFormat)) {
-            /* asfm */
-            if(transcode) {
-                size += 11;   /* 'wav' */
-            } else {
-                size += DMAPLEN(valarray[8]);
-            }
-        }
-        if(db_wantsmeta(pinfo->meta,metaSongDescription)) {
-            /* asdt */
-            if(transcode) {
-                size += 22;  /* 'wav audio file' */
-            } else {
-                size += DMAPLEN(valarray[29]);
-            }
-        }
-        if(db_wantsmeta(pinfo->meta,metaItemName))
-            /* minm */
-            size += DMAPLEN(valarray[3]);
-        if(valarray[34] && atoi(valarray[34]) && db_wantsmeta(pinfo->meta,metaSongDisabled))
-            /* asdb */
-            size += 9;
-        if(valarray[15] && atoi(valarray[15]) && db_wantsmeta(pinfo->meta,metaSongSampleRate))
-            /* assr */
-            size += 12;
-        if(valarray[17] && atoi(valarray[17]) && db_wantsmeta(pinfo->meta,metaSongSize))
-            /* assz */
-            size += 12;
-
-        /* In the old daap code, we always returned 0 for asst and assp
-         * (song start time, song stop time).  I don't know if this
-         * is required, so I'm going to disabled it
-         */
-
-        if(valarray[16] && atoi(valarray[16]) && db_wantsmeta(pinfo->meta, metaSongTime))
-            /* astm */
-            size += 12;
-        if(valarray[20] && atoi(valarray[20]) && db_wantsmeta(pinfo->meta, metaSongTrackCount))
-            /* astc */
-            size += 10;
-        if(valarray[19] && atoi(valarray[19]) && db_wantsmeta(pinfo->meta, metaSongTrackNumber))
-            /* astn */
-            size += 10;
-        if(valarray[25] && atoi(valarray[25]) && db_wantsmeta(pinfo->meta, metaSongUserRating))
-            /* asur */
-            size += 9;
-        if(valarray[18] && atoi(valarray[18]) && db_wantsmeta(pinfo->meta, metaSongYear))
-            /* asyr */
-            size += 10;
-        if(db_wantsmeta(pinfo->meta, metaContainerItemId))
-            /* mcti */
-            size += 12;
-        if((valarray[37]) && (strlen(valarray[37]) == 4) && 
-           db_wantsmeta(pinfo->meta,metaSongCodecType)) 
-            /* ascd */
-            size += 12;
-
-        if(db_wantsmeta(pinfo->meta,metaSongContentRating))
-            /* ascr */
-            size += 9;
-        if(db_wantsmeta(pinfo->meta,metaItunesHasVideo))
-            /* aeHV */
-            size += 9;
-
-        return size;
-        break;
-
-    default:
-        DPRINTF(E_LOG,L_DB|L_DAAP,"Unknown query type: %d\n",(int)pinfo->query_type);
-        return 0;
-    }
-    return 0;
-}
-
-int db_sql_build_dmap(DBQUERYINFO *pinfo, char **valarray, unsigned char *presult, int len) {
-    unsigned char *current = presult;
-    int transcode;
-    int samplerate=0;
-
-    switch(pinfo->query_type) {
-    case queryTypeBrowseArtists: /* simple 'mlit' entry */
-    case queryTypeBrowseAlbums:
-    case queryTypeBrowseGenres:
-    case queryTypeBrowseComposers:
-        return db_dmap_add_string(current,"mlit",valarray[0]);
-    case queryTypePlaylists:
-        /* do I want to include the mlit? */
-        current += db_dmap_add_container(current,"mlit",len - 8);
-        if(db_wantsmeta(pinfo->meta,metaItemId))
-            current += db_dmap_add_int(current,"miid",atoi(valarray[plID]));
-        current += db_dmap_add_int(current,"mimc",atoi(valarray[plItems]));
-        if(db_wantsmeta(pinfo->meta,metaItunesSmartPlaylist)) {
-            if(valarray[plType] && (atoi(valarray[plType]) == 1))
-                current += db_dmap_add_char(current,"aeSP",1);
-        }
-        if(db_wantsmeta(pinfo->meta,metaItemName))
-            current += db_dmap_add_string(current,"minm",valarray[plTitle]);
-        if((valarray[plType]) && (atoi(valarray[plType])==1) &&
-           db_wantsmeta(pinfo->meta, metaMPlaylistSpec))
-            current += db_dmap_add_string(current,"MSPS",valarray[plQuery]);
-        if(db_wantsmeta(pinfo->meta, metaMPlaylistType))
-            current += db_dmap_add_char(current,"MPTY",atoi(valarray[plType]));
-        break;
-    case queryTypeItems:
-    case queryTypePlaylistItems:  /* essentially the same query */
-        /* see if this is going to be transcoded */
-        transcode = plugin_ssc_should_transcode(pinfo->pwsc,valarray[37]);
-
-        /* Items that get changed by transcode:
-         *
-         * type:         item  8: changes to 'wav'
-         * description:  item 29: changes to 'wav audio file'
-         * bitrate:      item 15: guestimated, but doesn't change file size
-         *
-         * probably file size should change as well, but currently doesn't
-         */
-
-        current += db_dmap_add_container(current,"mlit",len-8);
-        if(db_wantsmeta(pinfo->meta, metaItemKind))
-            current += db_dmap_add_char(current,"mikd",(char)atoi(valarray[28]));
-        if(db_wantsmeta(pinfo->meta, metaSongDataKind))
-            current += db_dmap_add_char(current,"asdk",(char)atoi(valarray[27]));
-        if(EMIT(valarray[13]) && db_wantsmeta(pinfo->meta, metaSongDataURL))
-            current += db_dmap_add_string(current,"asul",valarray[13]);
-        if(EMIT(valarray[5]) && db_wantsmeta(pinfo->meta, metaSongAlbum))
-            current += db_dmap_add_string(current,"asal",valarray[5]);
-        if(EMIT(valarray[4]) && db_wantsmeta(pinfo->meta, metaSongArtist))
-            current += db_dmap_add_string(current,"asar",valarray[4]);
-        if(valarray[23] && atoi(valarray[23]) && db_wantsmeta(pinfo->meta, metaSongBPM))
-            current += db_dmap_add_short(current,"asbt",(short)atoi(valarray[23]));
-        if(valarray[14] && atoi(valarray[14]) && db_wantsmeta(pinfo->meta, metaSongBitRate)) {
-            if(transcode) {
-                if(valarray[15]) samplerate=atoi(valarray[15]);
-                if(samplerate) {
-                    current += db_dmap_add_short(current,"asbr",
-                                                 (short)(samplerate / 250 * 8));
-                }
-            } else {
-                current += db_dmap_add_short(current,"asbr",(short)atoi(valarray[14]));
-            }
-        }
-        if(EMIT(valarray[7]) && db_wantsmeta(pinfo->meta, metaSongComment))
-            current += db_dmap_add_string(current,"ascm",valarray[7]);
-        if(valarray[24] && atoi(valarray[24]) && db_wantsmeta(pinfo->meta,metaSongCompilation))
-            current += db_dmap_add_char(current,"asco",(char)atoi(valarray[24]));
-        if(EMIT(valarray[9]) && db_wantsmeta(pinfo->meta, metaSongComposer))
-            current += db_dmap_add_string(current,"ascp",valarray[9]);
-        if(EMIT(valarray[12]) && db_wantsmeta(pinfo->meta, metaSongGrouping))
-            current += db_dmap_add_string(current,"agrp",valarray[12]);
-        if(valarray[30] && atoi(valarray[30]) && db_wantsmeta(pinfo->meta, metaSongDateAdded))
-            current += db_dmap_add_int(current,"asda",(int)atoi(valarray[30]));
-        if(valarray[31] && atoi(valarray[31]) && db_wantsmeta(pinfo->meta,metaSongDateModified))
-            current += db_dmap_add_int(current,"asdm",(int)atoi(valarray[31]));
-        if(valarray[22] && atoi(valarray[22]) && db_wantsmeta(pinfo->meta, metaSongDiscCount))
-            current += db_dmap_add_short(current,"asdc",(short)atoi(valarray[22]));
-        if(valarray[21] && atoi(valarray[21]) && db_wantsmeta(pinfo->meta, metaSongDiscNumber))
-            current += db_dmap_add_short(current,"asdn",(short)atoi(valarray[21]));
-        if(EMIT(valarray[6]) && db_wantsmeta(pinfo->meta, metaSongGenre))
-            current += db_dmap_add_string(current,"asgn",valarray[6]);
-        if(db_wantsmeta(pinfo->meta,metaItemId))
-            current += db_dmap_add_int(current,"miid",(int)atoi(valarray[0]));
-        if(EMIT(valarray[8]) && db_wantsmeta(pinfo->meta,metaSongFormat)) {
-            if(transcode) {
-                current += db_dmap_add_string(current,"asfm","wav");
-            } else {
-                current += db_dmap_add_string(current,"asfm",valarray[8]);
-            }
-        }
-        if(EMIT(valarray[29]) && db_wantsmeta(pinfo->meta,metaSongDescription)) {
-            if(transcode) {
-                current += db_dmap_add_string(current,"asdt","wav audio file");
-            } else {
-                current += db_dmap_add_string(current,"asdt",valarray[29]);
-            }
-        }
-        if(EMIT(valarray[3]) && db_wantsmeta(pinfo->meta,metaItemName))
-            current += db_dmap_add_string(current,"minm",valarray[3]);
-        if(valarray[34] && atoi(valarray[34]) && db_wantsmeta(pinfo->meta,metaSongDisabled))
-            current += db_dmap_add_char(current,"asdb",(char)atoi(valarray[34]));
-        if(valarray[15] && atoi(valarray[15]) && db_wantsmeta(pinfo->meta,metaSongSampleRate))
-            current += db_dmap_add_int(current,"assr",atoi(valarray[15]));
-        if(valarray[17] && atoi(valarray[17]) && db_wantsmeta(pinfo->meta,metaSongSize))
-            current += db_dmap_add_int(current,"assz",atoi(valarray[17]));
-        if(valarray[16] && atoi(valarray[16]) && db_wantsmeta(pinfo->meta, metaSongTime))
-            current += db_dmap_add_int(current,"astm",atoi(valarray[16]));
-        if(valarray[20] && atoi(valarray[20]) && db_wantsmeta(pinfo->meta, metaSongTrackCount))
-            current += db_dmap_add_short(current,"astc",(short)atoi(valarray[20]));
-        if(valarray[19] && atoi(valarray[19]) && db_wantsmeta(pinfo->meta, metaSongTrackNumber))
-            current += db_dmap_add_short(current,"astn",(short)atoi(valarray[19]));
-        if(valarray[25] && atoi(valarray[25]) && db_wantsmeta(pinfo->meta, metaSongUserRating))
-            current += db_dmap_add_char(current,"asur",(char)atoi(valarray[25]));
-        if(valarray[18] && atoi(valarray[18]) && db_wantsmeta(pinfo->meta, metaSongYear))
-            current += db_dmap_add_short(current,"asyr",(short)atoi(valarray[18]));
-        if((valarray[37]) && (strlen(valarray[37]) == 4) && 
-           db_wantsmeta(pinfo->meta,metaSongCodecType))
-            current += db_dmap_add_literal(current,"ascd",valarray[37],4);
-        if(db_wantsmeta(pinfo->meta, metaContainerItemId))
-            current += db_dmap_add_int(current,"mcti",atoi(valarray[0]));
-        if(db_wantsmeta(pinfo->meta, metaItunesHasVideo))
-            current += db_dmap_add_char(current,"aeHV",atoi(valarray[39]));
-        if(db_wantsmeta(pinfo->meta, metaSongContentRating))
-                current += db_dmap_add_char(current,"ascr",atoi(valarray[40]));
-        return 0;
-        break;
-
-    default:
-        DPRINTF(E_LOG,L_DB|L_DAAP,"Unknown query type: %d\n",(int)pinfo->query_type);
-        return 0;
-    }
-    return 0;
-}
-
-int db_sql_atoi(const char *what) {
-    return what ? atoi(what) : 0;
-}
-char *db_sql_strdup(const char *what) {
-    return what ? (strlen(what) ? strdup(what) : NULL) : NULL;
-}
-
 void db_sql_build_m3ufile(SQL_ROW valarray, M3UFILE *pm3u) {
     memset(pm3u,0x00,sizeof(M3UFILE));
 
@@ -1799,7 +1463,7 @@ void db_sql_build_mp3file(SQL_ROW valarray, MP3FILE *pmp3) {
     pmp3->bitrate=db_sql_atoi(valarray[14]);
     pmp3->samplerate=db_sql_atoi(valarray[15]);
     pmp3->song_length=db_sql_atoi(valarray[16]);
-    pmp3->file_size=db_sql_atoi(valarray[17]);
+    pmp3->file_size=db_sql_atol(valarray[17]);
     pmp3->year=db_sql_atoi(valarray[18]);
     pmp3->track=db_sql_atoi(valarray[19]);
     pmp3->total_tracks=db_sql_atoi(valarray[20]);
@@ -1817,12 +1481,14 @@ void db_sql_build_mp3file(SQL_ROW valarray, MP3FILE *pmp3) {
     pmp3->time_played=db_sql_atoi(valarray[32]);
     pmp3->db_timestamp=db_sql_atoi(valarray[33]);
     pmp3->disabled=db_sql_atoi(valarray[34]);
-    pmp3->sample_count=db_sql_atoi(valarray[35]);
+    pmp3->sample_count=db_sql_atol(valarray[35]);
     pmp3->force_update=db_sql_atoi(valarray[36]);
     pmp3->codectype=db_sql_strdup(valarray[37]);
     pmp3->index=db_sql_atoi(valarray[38]);
     pmp3->has_video=db_sql_atoi(valarray[39]);
     pmp3->contentrating=db_sql_atoi(valarray[40]);
+    pmp3->bits_per_sample=db_sql_atoi(valarray[41]);
+    pmp3->album_artist=db_sql_strdup(valarray[42]);
 }
 
 /**
@@ -1899,7 +1565,7 @@ MP3FILE *db_sql_fetch_item(char **pe, int id) {
     db_sql_dispose_row();
 
     if ((db_sql_in_scan || db_sql_in_playlist_scan) && (!db_sql_reload)) {
-        db_sql_exec_fn(pe,E_FATAL,"INSERT INTO updated VALUES (%d)",id);
+        db_sql_exec_fn(pe,E_DBG,"INSERT INTO updated VALUES (%d)",id);
     }
 
     return pmp3;
@@ -1923,7 +1589,7 @@ MP3FILE *db_sql_fetch_path(char **pe, char *path, int index) {
      */
     if((db_sql_in_scan) && (db_sql_reload))
         return NULL;
-    
+
     /* not very portable, but works for sqlite */
     proper_path = _db_proper_path(path);
     query="select * from songs where path='%q' and idx=%d";
@@ -1949,7 +1615,7 @@ MP3FILE *db_sql_fetch_path(char **pe, char *path, int index) {
     db_sql_dispose_row();
 
     if ((db_sql_in_scan || db_sql_in_playlist_scan) && (!db_sql_reload)) {
-        db_sql_exec_fn(pe,E_FATAL,"INSERT INTO updated VALUES (%d)",pmp3->id);
+        db_sql_exec_fn(pe,E_DBG,"INSERT INTO updated VALUES (%d)",pmp3->id);
     }
 
     return pmp3;
@@ -1980,6 +1646,7 @@ void db_sql_dispose_item(MP3FILE *pmp3) {
     MAYBEFREE(pmp3->description);
     MAYBEFREE(pmp3->url);
     MAYBEFREE(pmp3->codectype);
+    MAYBEFREE(pmp3->album_artist);
     free(pmp3);
 }
 
