@@ -1,5 +1,5 @@
 /*
- * $Id: err.h 387 2004-11-13 08:05:27Z rpedde $
+ * $Id: err.h 1363 2006-08-25 03:37:03Z rpedde $
  * Error related routines
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -29,15 +29,15 @@
 #define __ERR_H__
 
 /** @anchor log_dests */
-#define LOGDEST_STDERR       0  /**< Log to stderr */
-#define LOGDEST_SYSLOG       1  /**< Log to syslog */
-#define LOGDEST_LOGFILE      2  /**< Log to logfile */
+#define LOGDEST_STDERR       1  /**< Log to stderr */
+#define LOGDEST_SYSLOG       2  /**< Log to syslog/eventviewer */
+#define LOGDEST_LOGFILE      4  /**< Log to logfile */
 
 /** @anchor log_levels */
 #define E_SPAM         10   /**< Logorrhea! */
 #define E_DBG           9   /**< Way too verbose */
 #define E_INF           5   /**< Good info, not too much spam */
-#define E_WARN          2   /**< Reasonably important, but not enough to log */
+#define E_WARN          2   /**< Goes in text log, but not syslog */
 #define E_LOG           1   /**< Something that should go in a log file */
 #define E_FATAL         0   /**< Log and force an exit */
 
@@ -54,43 +54,37 @@
 #define L_DAAP    0x00000200 /**< Generally daap related - main.c, daap.c, query.c */
 #define L_MAIN    0x00000400 /**< setup, teardown, signals - main.c */
 #define L_REND    0x00000800 /**< rendezvous -- rend-unix.c, rend-posix.c, etc */
-
+#define L_XML     0x00001000 /**< xml - xml-rpc.c */
+#define L_PARSE   0x00002000 /**< smart playlist parser */
+#define L_PLUG    0x00004000 /**< plugins */
+#define L_LOCK    0x00008000 /**< semaphore locks */
 #define L_MISC    0x80000000 /**< anything else */
 
-extern int err_debuglevel;
+#ifndef TRUE
+# define TRUE 1
+# define FALSE 0
+#endif
 
 extern void err_log(int level, unsigned int cat, char *fmt, ...);
-extern void err_setdest(char *app, int destination);
+extern void err_reopen(void); /** rotate logfile */
+extern void err_setdest(int destination);
+extern int err_getdest(void);
+extern void err_setlevel(int level);
+extern int err_getlevel(void);
 extern int err_setdebugmask(char *list);
+extern int err_setlogfile(char *file);
+extern int err_settruncate(int truncate);
+
 /**
  * Print a debugging or log message
  */
-#ifdef DEBUG
-# define DPRINTF(level, cat, fmt, arg...)				\
-    { err_log(level,cat,"%s, %d: ",__FILE__,__LINE__); err_log(level,cat,fmt,##arg); }
-#else
-# define DPRINTF(level, cat, fmt, arg...)	\
-    { err_log(level,cat,fmt,##arg); }
+
+#define DPRINTF err_log
+
+#ifdef ERR_LEAN
+# define os_syslog(a,b)
+# define os_opensyslog(a)
+# define os_closesyslog(a)
 #endif
 
-#ifdef DEBUG_MEMORY
-# ifndef __IN_ERR__
-#  define malloc(x) err_malloc(__FILE__,__LINE__,x)
-#  define strdup(x) err_strdup(__FILE__,__LINE__,x)
-#  define free(x) err_free(__FILE__,__LINE__,x)
-# endif /* __IN_ERR__ */
-
-# define MEMNOTIFY(x) err_notify(__FILE__,__LINE__,x)
-
-extern void *err_malloc(char *file, int line, size_t size);
-extern char *err_strdup(char *file, int line, const char *str);
-extern void err_free(char *file, int line, void *ptr);
-extern void err_notify(char *file, int line, void *ptr);
-extern void err_leakcheck(void);
-#else
-/**
- * Notify the leak checking system of externally allocated memory.
- */
-# define MEMNOTIFY(x)
-#endif /* DEBUG_MEMORY */
 #endif /* __ERR_H__ */
