@@ -26,31 +26,48 @@
 #define _XOPEN_SOURCE 500  /** unix98?  pthread_once_t, etc */
 
 #include <errno.h>
+#include <fcntl.h>
 #include <pthread.h>
-#include <regex.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #include <sys/stat.h>
 #include <sys/types.h>
-
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include "daapd.h"
 #include "conf.h"
+#include "configfile.h"
 #include "db-generic.h"
-#include "dispatch.h"
 #include "err.h"
+#include "io.h"
 #include "os.h"
 #include "plugin.h"
 #include "rend.h"
+#include "restart.h"
 #include "smart-parser.h"
 #include "xml-rpc.h"
 #include "webserver.h"
 #include "ff-plugins.h"
+#include "io.h"
 
 typedef struct tag_pluginentry {
     void *phandle;
-    regex_t regex;
     PLUGIN_INFO *pinfo;
     struct tag_pluginentry *next;
 } PLUGIN_ENTRY;
@@ -73,55 +90,7 @@ void _plugin_unlock(void);
 int _plugin_error(char **pe, int error, ...);
 void _plugin_free(int *pi);
 void _plugin_recalc_codecs(void);
-
-/* webserver helpers */
-char *pi_ws_uri(WS_CONNINFO *pwsc);
-void pi_ws_close(WS_CONNINFO *pwsc);
-int pi_ws_fd(WS_CONNINFO *pwsc);
-
-/* misc helpers */
-char *pi_server_ver(void);
-int pi_server_name(char *, int *);
-void pi_log(int, char *, ...);
-
-/* db helpers */
-int pi_db_count(void);
-int pi_db_enum_start(char **pe, DB_QUERY *pinfo);
-int pi_db_enum_fetch_row(char **pe, char ***row, DB_QUERY *pinfo);
-int pi_db_enum_end(char **pe);
-void pi_db_enum_dispose(char **pe, DB_QUERY *pinfo);
-void pi_stream(WS_CONNINFO *pwsc, char *id);
-
-void pi_conf_dispose_string(char *str);
-
-PLUGIN_INPUT_FN pi = {
-    pi_ws_uri,
-    pi_ws_close,
-    ws_returnerror,
-    ws_getvar,
-    ws_writefd,
-    ws_addresponseheader,
-    ws_emitheaders,
-    pi_ws_fd,
-    ws_getrequestheader,
-    ws_writebinary,
-
-    pi_server_ver,
-    pi_server_name,
-    pi_log,
-    plugin_ssc_should_transcode,
-
-    pi_db_count,
-    pi_db_enum_start,
-    pi_db_enum_fetch_row,
-    pi_db_enum_end,
-    pi_db_enum_dispose,
-    pi_stream,
-
-    conf_alloc_string,
-    pi_conf_dispose_string
-};
-
+int _plugin_ssc_transcode(WS_CONNINFO *pwsc, MP3FILE *pmp3, int offset, int headers);
 
 /**
  * initialize stuff for plugins
@@ -150,8 +119,8 @@ int plugin_deinit(void) {
 }
 
 /**
- * return the error 
- * 
+ * return the error
+ *
  * @param pe buffer to store the error string in
  * @param error error to return
  * @returns the specified error,to thelp with returns
@@ -198,7 +167,7 @@ void _plugin_recalc_codecs(void) {
     if(!_plugin_ssc_codecs) {
         DPRINTF(E_FATAL,L_PLUG,"_plugin_recalc_codecs: malloc\n");
     }
-    
+
     memset(_plugin_ssc_codecs,0,size+1);
 
     ppi = _plugin_list.next;
@@ -217,6 +186,35 @@ void _plugin_recalc_codecs(void) {
 
 }
 
+/**
+ * plugin_get_description
+ */
+char *plugin_get_description(void *which) {
+    PLUGIN_ENTRY *ppi = (PLUGIN_ENTRY *)which;
+
+    return ppi->pinfo->server;
+}
+
+
+/**
+ * walk through the loaded plugin list
+ */
+void *plugin_enum(void *where) {
+    PLUGIN_ENTRY *ppi = (PLUGIN_ENTRY *)where;
+
+    if(!ppi) {
+        // _plugin_readlock();
+        return (void*) _plugin_list.next;
+    }
+
+    if(!ppi->next) {
+        // _plugin_unlock();
+        return NULL;
+    }
+
+    return (void*) ppi->next;
+}
+
 /**
  * load a specified plugin.
  *
@@ -228,11 +226,11 @@ void _plugin_recalc_codecs(void) {
 int plugin_load(char **pe, char *path) {
     PLUGIN_ENTRY *ppi;
     void *phandle;
-    PLUGIN_INFO *(*info_func)(PLUGIN_INPUT_FN *);
+    PLUGIN_INFO *(*info_func)(void);
     PLUGIN_INFO *pinfo;
 
     DPRINTF(E_DBG,L_PLUG,"Attempting to load plugin %s\n",path);
-    
+
     phandle = os_loadlib(pe, path);
     if(!phandle) {
         DPRINTF(E_INF,L_PLUG,"Couldn't get lib handle for %s\n",path);
@@ -244,15 +242,15 @@ int plugin_load(char **pe, char *path) {
 
     ppi->phandle = phandle;
 
-    info_func = (PLUGIN_INFO*(*)(PLUGIN_INPUT_FN*)) os_libfunc(pe, phandle,"plugin_info");
+    info_func = (PLUGIN_INFO*(*)(void)) os_libfunc(pe, phandle,"plugin_info");
     if(info_func == NULL) {
         DPRINTF(E_INF,L_PLUG,"Couldn't get info_func for %s\n",path);
         os_unload(phandle);
         free(ppi);
         return PLUGIN_E_BADFUNCS;
     }
-    
-    pinfo = info_func(&pi);
+
+    pinfo = info_func();
     ppi->pinfo = pinfo;
 
     if(!pinfo) {
@@ -262,11 +260,12 @@ int plugin_load(char **pe, char *path) {
         return PLUGIN_E_NOLOAD;
     }
 
-    if(pinfo->type & PLUGIN_OUTPUT) {
-        /* build the regex */
-        if(regcomp(&ppi->regex,pinfo->url,REG_EXTENDED | REG_NOSUB)) {
-            DPRINTF(E_LOG,L_PLUG,"Bad regex in %s: %s\n",path,pinfo->url);
-        }
+    if(pinfo->version != PLUGIN_VERSION) {
+        DPRINTF(E_INF,L_PLUG,"Plugin is too old: version %d, expecting %d\n",
+                pinfo->version, PLUGIN_VERSION);
+        os_unload(phandle);
+        free(ppi);
+        return PLUGIN_E_NOLOAD;
     }
 
     DPRINTF(E_INF,L_PLUG,"Loaded plugin %s (%s)\n",path,pinfo->server);
@@ -278,7 +277,7 @@ int plugin_load(char **pe, char *path) {
 
     ppi->next = _plugin_list.next;
     _plugin_list.next = ppi;
-    
+
     _plugin_recalc_codecs();
     return PLUGIN_E_SUCCESS;
 }
@@ -287,32 +286,29 @@ int plugin_load(char **pe, char *path) {
  * check to see if we want to dispatch a particular url
  *
  * @param pwsc the connection info (including uri) to check
- * @returns TRUE if we want to handle it 
+ * @returns TRUE if we want to handle it
  */
 int plugin_url_candispatch(WS_CONNINFO *pwsc) {
     PLUGIN_ENTRY *ppi;
-    
-    DPRINTF(E_DBG,L_PLUG,"Entering candispatch\n");
 
     ppi = _plugin_list.next;
     while(ppi) {
         if(ppi->pinfo->type & PLUGIN_OUTPUT) {
-            if(!regexec(&ppi->regex,pwsc->uri,0,NULL,0)) {
-                /* we have a winner */
+            if((ppi->pinfo->output_fns)->can_handle(pwsc)) {
                 return TRUE;
             }
         }
         ppi = ppi->next;
     }
+
     return FALSE;
 }
 
-
 /**
  * actually DISPATCH the hander we said we wanted
  *
  * @param pwsc the connection info (including uri) to check
- * @returns TRUE if we want to handle it 
+ * @returns TRUE if we want to handle it
  */
 void plugin_url_handle(WS_CONNINFO *pwsc) {
     PLUGIN_ENTRY *ppi;
@@ -321,9 +317,9 @@ void plugin_url_handle(WS_CONNINFO *pwsc) {
     ppi = _plugin_list.next;
     while(ppi) {
         if(ppi->pinfo->type & PLUGIN_OUTPUT) {
-            if(!regexec(&ppi->regex,pwsc->uri,0,NULL,0)) {
+            if((ppi->pinfo->output_fns)->can_handle(pwsc)) {
                 /* we have a winner */
-                DPRINTF(E_DBG,L_PLUG,"Dispatching %s to %s\n", pwsc->uri,
+                DPRINTF(E_DBG,L_PLUG,"Dispatching %s to %s\n", ws_uri(pwsc),
                         ppi->pinfo->server);
 
                 /* so functions must be a tag_plugin_output_fn */
@@ -338,7 +334,7 @@ void plugin_url_handle(WS_CONNINFO *pwsc) {
     /* should 500 here or something */
     ws_returnerror(pwsc, 500, "Can't find plugin handler");
     return;
-} 
+}
 
 /**
  * walk through the plugins and register whatever rendezvous
@@ -377,7 +373,7 @@ int plugin_rend_register(char *name, int port, char *iface, char *txt) {
  * @param pwsc the connection info (including uri) to check
  * @param username user attempting to login
  * @param pw password attempting
- * @returns TRUE if we want to handle it 
+ * @returns TRUE if we want to handle it
  */
 int plugin_auth_handle(WS_CONNINFO *pwsc, char *username, char *pw) {
     PLUGIN_ENTRY *ppi;
@@ -387,9 +383,9 @@ int plugin_auth_handle(WS_CONNINFO *pwsc, char *username, char *pw) {
     ppi = _plugin_list.next;
     while(ppi) {
         if(ppi->pinfo->type & PLUGIN_OUTPUT) {
-            if(!regexec(&ppi->regex,pwsc->uri,0,NULL,0)) {
+            if((ppi->pinfo->output_fns)->can_handle(pwsc)) {
                 /* we have a winner */
-                DPRINTF(E_DBG,L_PLUG,"Dispatching %s to %s\n", pwsc->uri,
+                DPRINTF(E_DBG,L_PLUG,"Dispatching %s to %s\n", ws_uri(pwsc),
                         ppi->pinfo->server);
 
                 /* so functions must be a tag_plugin_output_fn */
@@ -430,60 +426,11 @@ void plugin_event_dispatch(int event_id, int intval, void *vp, int len) {
     }
 }
 
-/**
- * check to see if we can transcode
- * 
- * @param codec the codec we are trying to serve
- * @returns TRUE if we can transcode, FALSE otherwise
- */
-int plugin_ssc_should_transcode(WS_CONNINFO *pwsc, char *codec) {
-    int result;
-    char *native_codecs=NULL;
-    char *user_agent=NULL;
-    
-    if(!codec) {
-        DPRINTF(E_LOG,L_PLUG,"testing transcode on null codec?\n");
-        return FALSE;
-    }
-
-    if(pwsc) {
-        /* see if the headers give us any guidance */
-        native_codecs = ws_getrequestheader(pwsc,"accept-codecs");
-        if(!native_codecs) {
-            user_agent = ws_getrequestheader(pwsc,"user-agent");
-            if(user_agent) {
-                if(strncmp(user_agent,"iTunes",6)==0) {
-                    native_codecs = "mpeg,mp4a,wav,mp4v,alac";
-                } else if(strncmp(user_agent,"Roku",4)==0) {
-                    native_codecs = "mpeg,mp4a,wav,wma";
-                }
-            }
-        }
-    }
-
-    if(!native_codecs) {
-        native_codecs = "mpeg,wav";
-    }
-
-    /* can't transcode it if we can't transcode it */
-    if(!_plugin_ssc_codecs)
-        return FALSE;
-
-    if(strstr(native_codecs,codec))
-        return FALSE;
-
-    result = FALSE;
-    if(strstr(_plugin_ssc_codecs,codec)) {
-        result = TRUE;
-    }
-    return result;
-}
-
 
 /**
  * stupid helper to copy transcode stream to the fd
  */
-int _plugin_ssc_copy(WS_CONNINFO *pwsc, PLUGIN_TRANSCODE_FN *pfn, 
+int __plugin_ssc_copy(WS_CONNINFO *pwsc, PLUGIN_TRANSCODE_FN *pfn,
                      void *vp,int offset) {
     int bytes_read;
     int bytes_to_read;
@@ -499,7 +446,7 @@ int _plugin_ssc_copy(WS_CONNINFO *pwsc, PLUGIN_TRANSCODE_FN *pfn,
         bytes_read = pfn->ssc_read(vp,buffer,bytes_to_read);
         if(bytes_read <= 0)
             return bytes_read;
-        
+
         offset -= bytes_read;
     }
 
@@ -510,8 +457,11 @@ int _plugin_ssc_copy(WS_CONNINFO *pwsc, PLUGIN_TRANSCODE_FN *pfn,
         }
     }
 
-    if(bytes_read < 0)
+    /*
+    if(bytes_read < 0) {
         return bytes_read;
+    }
+    */
 
     return total_bytes_read;
 }
@@ -526,7 +476,7 @@ int _plugin_ssc_copy(WS_CONNINFO *pwsc, PLUGIN_TRANSCODE_FN *pfn,
  * @param duration time in ms
  * @returns bytes transferred, or -1 on error
  */
-int plugin_ssc_transcode(WS_CONNINFO *pwsc, char *file, char *codec, int duration, int offset, int headers) {
+int plugin_ssc_transcode(WS_CONNINFO *pwsc, MP3FILE *pmp3, int offset, int headers) {
     PLUGIN_ENTRY *ppi, *ptc=NULL;
     PLUGIN_TRANSCODE_FN *pfn = NULL;
     void *vp_ssc;
@@ -534,12 +484,10 @@ int plugin_ssc_transcode(WS_CONNINFO *pwsc, char *file, char *codec, int duratio
     int result = -1;
 
     /* first, find the plugin that will do the conversion */
-
-
     ppi = _plugin_list.next;
     while((ppi) && (!pfn)) {
         if(ppi->pinfo->type & PLUGIN_TRANSCODE) {
-            if(strstr(ppi->pinfo->codeclist,codec)) {
+            if(strstr(ppi->pinfo->codeclist,pmp3->codectype)) {
                 ptc = ppi;
                 pfn = ppi->pinfo->transcode_fns;
             }
@@ -548,12 +496,12 @@ int plugin_ssc_transcode(WS_CONNINFO *pwsc, char *file, char *codec, int duratio
     }
 
     if(pfn) {
-        DPRINTF(E_DBG,L_PLUG,"Transcoding %s with %s\n",file,
+        DPRINTF(E_DBG,L_PLUG,"Transcoding %s with %s\n",pmp3->path,
                 ptc->pinfo->server);
 
         vp_ssc = pfn->ssc_init();
         if(vp_ssc) {
-            if(pfn->ssc_open(vp_ssc,file,codec,duration)) {
+            if(pfn->ssc_open(vp_ssc,pmp3)) {
                 /* start reading and throwing */
                 if(headers) {
                     ws_addresponseheader(pwsc,"Content-Type","audio/wav");
@@ -561,21 +509,21 @@ int plugin_ssc_transcode(WS_CONNINFO *pwsc, char *file, char *codec, int duratio
                     if(!offset) {
                         ws_writefd(pwsc,"HTTP/1.1 200 OK\r\n");
                     } else {
-                        ws_addresponseheader(pwsc,"Content-Range","bytes %ld-*/*",
+                        ws_addresponseheader(pwsc,"Content-Range",
+                                             "bytes %ld-*/*",
                                              (long)offset);
                         ws_writefd(pwsc,"HTTP/1.1 206 Partial Content\r\n");
                     }
                     ws_emitheaders(pwsc);
                 }
 
-
                 /* start reading/writing */
-                result = _plugin_ssc_copy(pwsc,pfn,vp_ssc,offset);
+                result = __plugin_ssc_copy(pwsc,pfn,vp_ssc,offset);
                 post_error = 0;
                 pfn->ssc_close(vp_ssc);
             } else {
                 DPRINTF(E_LOG,L_PLUG,"Error opening %s for ssc: %s\n",
-                        file,pfn->ssc_error(vp_ssc));
+                        pmp3->path,pfn->ssc_error(vp_ssc));
             }
             pfn->ssc_deinit(vp_ssc);
         } else {
@@ -592,160 +540,73 @@ int plugin_ssc_transcode(WS_CONNINFO *pwsc, char *file, char *codec, int duratio
     return result;
 }
 
-/* plugin wrappers for utility functions & stuff
- * 
- * these functions need to be wrapped so we can maintain a stable
- * interface to older plugins even if we get newer functions or apis
- * upstream... it's a binary compatibility layer.
- */
-char *pi_ws_uri(WS_CONNINFO *pwsc) {
-    return pwsc->uri;
-}
-
-void pi_ws_close(WS_CONNINFO *pwsc) {
-    pwsc->close=1;
-}
-
-int pi_ws_fd(WS_CONNINFO *pwsc) {
-    return pwsc->fd;
-}
-
-void pi_log(int level, char *fmt, ...) {
-    char buf[256];
-    va_list ap;
-
-    va_start(ap,fmt);
-    vsnprintf(buf,sizeof(buf),fmt,ap);
-    va_end(ap);
-    
-    DPRINTF(level,L_PLUG,"%s",buf);
-}
-
-char *pi_server_ver(void) {
-    return VERSION;
-}
-
-int pi_server_name(char *name, int *len) {
-    char *servername;
-
-    servername = conf_get_servername();
-    if((servername) && (strlen(servername) < (size_t)len)) {
-        strcpy(name,servername);
-    } else {
-        if((size_t)len > strlen("Firefly Media Server"))
-            strcpy(name,"Firefly Media Server");
-    }
-
-    free(servername);
-    return CONF_E_SUCCESS;
-}
-
-int pi_db_count(void) {
-    int count;
-    db_get_song_count(NULL, &count);
-
-    return count;
-}
-
-int pi_db_enum_start(char **pe, DB_QUERY *pinfo) {
-    DBQUERYINFO *pqi;
+int plugin_ssc_should_transcode(WS_CONNINFO *pwsc, char *codec) {
     int result;
-    
-    pqi = (DBQUERYINFO*)malloc(sizeof(DBQUERYINFO));
-    if(!pqi) {
-        if(pe) *pe = strdup("Malloc error");
-        return DB_E_MALLOC;
-    }
-    memset(pqi,0,sizeof(DBQUERYINFO));
-    pinfo->priv = (void*)pqi;
+    char *native_codecs=NULL;
+    char *user_agent=NULL;
+    char *never_transcode = NULL;
+    char *always_transcode = NULL;
 
-    if(pinfo->filter) {
-        pqi->pt = sp_init();
-        if(!sp_parse(pqi->pt,pinfo->filter,pinfo->filter_type)) {
-            DPRINTF(E_LOG,L_PLUG,"Ignoring bad query (%s): %s\n",
-                    pinfo->filter,sp_get_error(pqi->pt));
-            sp_dispose(pqi->pt);
-            pqi->pt = NULL;
+    ASSERT((pwsc) && (codec));
+
+    if(!pwsc)
+        return FALSE;
+
+    if(!codec) {
+        DPRINTF(E_LOG,L_PLUG,"testing transcode on null codec?\n");
+        return FALSE;
+    }
+
+    never_transcode = conf_alloc_string("general","never_transcode",NULL);
+    if(never_transcode) {
+        if(strstr(never_transcode,codec)) {
+            free(never_transcode);
+            return FALSE;
+        }
+        free(never_transcode);
+    }
+
+    if(pwsc) {
+        /* see if the headers give us any guidance */
+        native_codecs = ws_getrequestheader(pwsc,"accept-codecs");
+        if(!native_codecs) {
+            user_agent = ws_getrequestheader(pwsc,"user-agent");
+            if(user_agent) {
+                if(strncmp(user_agent,"iTunes",6)==0) {
+                    native_codecs = "mpeg,mp4a,wav,mp4v,alac";
+                } else if(strncmp(user_agent,"Roku",4)==0) {
+                    native_codecs = "mpeg,mp4a,wav,wma";
+                } else if(strncmp(user_agent,"Hifidelio",9)==0) {
+                    return FALSE;
+                }
+            }
         }
     }
 
-    if((pinfo->limit) || (pinfo->offset)) {
-        pqi->index_low = pinfo->offset;
-        pqi->index_high = pinfo->offset + pinfo->limit - 1;
-        if(pqi->index_high < pqi->index_low)
-            pqi->index_high = 9999999;
-
-        pqi->index_type = indexTypeSub;
-    } else {
-        pqi->index_type = indexTypeNone;
+    if(!native_codecs) {
+        native_codecs = "mpeg,wav";
     }
 
-    pqi->want_count = 1;
+    /* can't transcode it if we can't transcode it */
+    if(!_plugin_ssc_codecs)
+        return FALSE;
 
-    switch(pinfo->query_type) {
-    case QUERY_TYPE_PLAYLISTS:
-        pqi->query_type = queryTypePlaylists;
-        break;
-    case QUERY_TYPE_DISTINCT:
-        if((strcmp(pinfo->distinct_field,"artist") == 0)) {
-            pqi->query_type = queryTypeBrowseArtists;
-        } else if((strcmp(pinfo->distinct_field,"genre") == 0)) {
-            pqi->query_type = queryTypeBrowseGenres;
-        } else if((strcmp(pinfo->distinct_field,"album") == 0)) {
-            pqi->query_type = queryTypeBrowseAlbums;
-        } else if((strcmp(pinfo->distinct_field,"composer") == 0)) {
-            pqi->query_type = queryTypeBrowseComposers;
-        } else {
-            if(pe) *pe = strdup("Unsupported browse type");
-            if(pqi->pt)
-                sp_dispose(pqi->pt);
-            pqi->pt = NULL;
-            return -1; /* not really a db error for this */
+    always_transcode = conf_alloc_string("general","always_transcode",NULL);
+    if(always_transcode) {
+        if(strstr(always_transcode,codec)) {
+            free(always_transcode);
+            return TRUE;
         }
-        break;
-    case QUERY_TYPE_ITEMS:
-    default:
-        pqi->query_type = queryTypePlaylistItems;
-        pqi->correct_order = conf_get_int("scan","correct_order",1);
-        break;
+        free(always_transcode);
     }
 
-    pqi->playlist_id = pinfo->playlist_id;
-    result =  db_enum_start(pe, pqi);
-    pinfo->totalcount = pqi->specifiedtotalcount;
+    if(strstr(native_codecs,codec))
+        return FALSE;
 
-    return DB_E_SUCCESS;
-}
-
-int pi_db_enum_fetch_row(char **pe, char ***row, DB_QUERY *pinfo) {
-    return db_enum_fetch_row(pe, (PACKED_MP3FILE*)row, 
-                             (DBQUERYINFO*)pinfo->priv);
-}
-
-int pi_db_enum_end(char **pe) {
-    return db_enum_end(pe);
-}
-
-void pi_stream(WS_CONNINFO *pwsc, char *id) {
-    dispatch_stream_id(pwsc, 0, id);
-    return;
-}
-
-void pi_db_enum_dispose(char **pe, DB_QUERY *pinfo) {
-    DBQUERYINFO *pqi;
-
-    if(!pinfo)
-        return;
-
-    if(pinfo->priv) {
-        pqi = (DBQUERYINFO *)pinfo->priv;
-        if(pqi->pt) {
-            sp_dispose(pqi->pt);
-            pqi->pt = NULL;
-        }
+    result = FALSE;
+    if(strstr(_plugin_ssc_codecs,codec)) {
+        result = TRUE;
     }
+    return result;
 }
 
-void pi_conf_dispose_string(char *str) {
-    free(str);
-}
