@@ -1,5 +1,5 @@
 /*
- * $Id: rend-unix.h 173 2004-01-20 04:41:20Z rpedde $
+ * $Id: rend-unix.h 801 2006-03-05 08:09:27Z rpedde $
  * General unix rendezvous routines
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -22,13 +22,18 @@
 #ifndef _REND_UNIX_H_
 #define _REND_UNIX_H_
 
-#define MAX_NAME_LEN 256
+#define MAX_NAME_LEN 200
+/* Is there a posixly correct constant for this? */
+#define MAX_IFACE_NAME_LEN 20
+#define MAX_TEXT_LEN 255
 
 typedef struct tag_rend_message {
     int cmd;
     int port;
     char name[MAX_NAME_LEN];
     char type[MAX_NAME_LEN];
+    char iface[MAX_IFACE_NAME_LEN];
+    char txt[MAX_TEXT_LEN];
 } REND_MESSAGE;
 
 #define REND_MSG_TYPE_REGISTER     0
