@@ -1,5 +1,5 @@
 /*
- * $Id: rend.h 173 2004-01-20 04:41:20Z rpedde $
+ * $Id: rend.h 801 2006-03-05 08:09:27Z rpedde $
  * Rendezvous stuff
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -25,7 +25,7 @@
 extern int rend_init(char *user);
 extern int rend_running(void);
 extern int rend_stop(void);
-extern int rend_register(char *name, char *type, int port);
+extern int rend_register(char *name, char *type, int port, char *iface, char *txt);
 extern int rend_unregister(char *name, char *type, int port);
 
 #endif /* _REND_H_ */
