@@ -1,5 +1,5 @@
 /*
- * $Id: scan-aac.h 616 2005-05-29 23:44:23Z rpedde $
+ * $Id: scan-aac.h 1622 2007-07-31 04:34:33Z rpedde $
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
  *
@@ -21,7 +21,9 @@
 #ifndef _SCAN_AAC_H_
 #define _SCAN_AAC_H_
 
-extern off_t scan_aac_drilltoatom(FILE *aac_fp, char *atom_path, unsigned int *atom_length);
-extern long scan_aac_findatom(FILE *fin, long max_offset, char *which_atom, unsigned int *atom_size);
+#include "io.h"
+
+extern uint64_t scan_aac_drilltoatom(IOHANDLE hfile, char *atom_path, unsigned int *atom_length);
+extern uint64_t scan_aac_findatom(IOHANDLE hfile, uint64_t max_offset, char *which_atom, unsigned int *atom_size);
 
 #endif
