@@ -1,5 +1,5 @@
 /*
- * $Id: scan-mp3.c 1375 2006-09-01 04:16:51Z rpedde $
+ * $Id: scan-mp3.c 1489 2007-01-18 06:07:26Z rpedde $
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
  *
@@ -26,13 +26,16 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <id3tag.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
-#include "conf.h"
 #include "daapd.h"
+#include "conf.h"
 #include "err.h"
 #include "mp3-scanner.h"
 
@@ -211,7 +214,7 @@ char *scan_winamp_genre[] = {
     "Samba",
     "Folklore",           // 115
     "Ballad",
-    "Powder Ballad",
+    "Power Ballad",
     "Rhythmic Soul",
     "Freestyle",
     "Duet",               // 120
@@ -306,7 +309,7 @@ int scan_mp3_get_mp3tags(char *file, MP3FILE *pmp3) {
         DPRINTF(E_WARN,L_SCAN,"Cannot open %s\n",file);
         return FALSE;
     }
-         
+
     pid3file=id3_file_fdopen(mp3_fd,ID3_FILE_MODE_READONLY);
     if(!pid3file) {
         DPRINTF(E_WARN,L_SCAN,"Cannot open %s\n",file);
@@ -360,6 +363,9 @@ int scan_mp3_get_mp3tags(char *file, MP3FILE *pmp3) {
                     utf8_text=(char *)id3_ucs4_utf8duplicate(native_text);
                 }
 
+                if(utf8_text)
+                    mem_register(utf8_text,0);
+
                 if(!strcmp(pid3frame->id,"TIT2")) { /* Title */
                     used=1;
                     pmp3->title = utf8_text;
@@ -481,6 +487,9 @@ int scan_mp3_get_mp3tags(char *file, MP3FILE *pmp3) {
             native_text=id3_field_getstring(&pid3frame->fields[2]);
             if(native_text) {
                 utf8_text=(char*)id3_ucs4_utf8duplicate(native_text);
+                if(utf8_text)
+                    mem_register(utf8_text,0);
+
                 if((utf8_text) && (strncasecmp(utf8_text,"iTun",4) != 0)) {
                     /* it's a real comment */
                     if(utf8_text)
@@ -492,6 +501,7 @@ int scan_mp3_get_mp3tags(char *file, MP3FILE *pmp3) {
                             free(pmp3->comment);
                         utf8_text=(char*)id3_ucs4_utf8duplicate(native_text);
                         if(utf8_text) {
+                            mem_register(utf8_text,0);
                             pmp3->comment=utf8_text;
                         }
                     }
@@ -827,7 +837,7 @@ int scan_mp3_get_mp3fileinfo(char *file, MP3FILE *pmp3) {
         if(ferror(infile)) {
             DPRINTF(E_LOG,L_SCAN,"Error reading: %s\n",strerror(errno));
         } else {
-            DPRINTF(E_LOG,L_SCAN,"Short file: %s\n",file);
+            DPRINTF(E_INF,L_SCAN,"Bad mp3 file? (short read): %s\n",file);
         }
         fclose(infile);
         return FALSE;
@@ -858,7 +868,7 @@ int scan_mp3_get_mp3fileinfo(char *file, MP3FILE *pmp3) {
         fseek(infile,fp_size,SEEK_SET);
         DPRINTF(E_DBG,L_SCAN,"Reading in new block at %d\n",(int)fp_size);
         if(fread(buffer,1,sizeof(buffer),infile) < sizeof(buffer)) {
-            DPRINTF(E_LOG,L_SCAN,"Short read: %s\n",file);
+            DPRINTF(E_INF,L_SCAN,"Bad mp3 file? (Short read): %s\n",file);
             fclose(infile);
             return TRUE;
         }
