@@ -1,5 +1,5 @@
 /*
- * $Id: scan-wma.c 1373 2006-08-31 00:37:16Z rpedde $
+ * $Id: scan-wma.c 1484 2007-01-17 01:06:16Z rpedde $
  * WMA metatag parsing
  *
  * Copyright (C) 2005 Ron Pedde (ron@pedde.com)
@@ -25,10 +25,14 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "daapd.h"
 #include "mp3-scanner.h"
 #include "restart.h"
 #include "err.h"
@@ -235,8 +239,8 @@ typedef struct tag_wma_header {
     char reserved1;
     char reserved2;
 } _PACKED WMA_HEADER;
-                          
-                          
+
+
 typedef struct tag_wma_subheader {
     unsigned char objectid[16];
     long long size;
@@ -262,7 +266,7 @@ typedef struct tag_wma_header_extension {
 
 /*
  * Forwards
- */ 
+ */
 WMA_GUID *wma_find_guid(unsigned char *guid);
 unsigned short int wma_convert_short(unsigned char *src);
 unsigned int wma_convert_int(unsigned char *src);
@@ -322,7 +326,7 @@ int wma_file_read_utf16(int fd, int len, char **utf8) {
     if(!utf16)
         return 0;
 
-    if(r_read(fd,utf16,len) != len) 
+    if(r_read(fd,utf16,len) != len)
         return 0;
 
     out = wma_utf16toutf8(utf16,len);
@@ -353,6 +357,7 @@ int wma_parse_header_extension(int fd, int size, MP3FILE *pmp3) {
     if(r_read(fd,&he,sizeof(he)) != sizeof(he))
         return FALSE;
 
+    he.data_size  = wma_convert_int((unsigned char *)&he.data_size);
     bytes_left = he.data_size;
     DPRINTF(E_DBG,L_SCAN,"Found header ext of %ld (%ld) bytes\n",he.data_size,size);
 
@@ -363,6 +368,7 @@ int wma_parse_header_extension(int fd, int size, MP3FILE *pmp3) {
         if(r_read(fd,&sh,sizeof(sh)) != sizeof(sh))
             return FALSE;
 
+        sh.size = wma_convert_ll((unsigned char *)&sh.size);
         pguid = wma_find_guid(sh.objectid);
         if(!pguid) {
             DPRINTF(E_DBG,L_SCAN,"  Unknown ext subheader: %02hhx%02hhx"
@@ -385,12 +391,12 @@ int wma_parse_header_extension(int fd, int size, MP3FILE *pmp3) {
             }
         }
 
-        DPRINTF(E_DBG,L_SCAN,"  Size: %ld\n",sh.size);
+        DPRINTF(E_DBG,L_SCAN,"  Size: %lld\n",sh.size);
         if(sh.size <= sizeof(sh))
             return TRUE; /* guess we're done! */
 
-        bytes_left -= sh.size;
-        lseek(fd,current + sh.size,SEEK_SET);
+        bytes_left -= (long)sh.size;
+        lseek(fd,current + (long)sh.size,SEEK_SET);
     }
 
     return TRUE;
@@ -415,7 +421,7 @@ int wma_parse_stream_properties(int fd, int size, MP3FILE *pmp3) {
     if(!pguid)
         return TRUE;
 
-    if(strcmp(pguid->name,"ASF_Audio_Media") != 0) 
+    if(strcmp(pguid->name,"ASF_Audio_Media") != 0)
         return TRUE;
 
     /* it is an audio stream... find codec.  The Type-Specific
@@ -426,7 +432,7 @@ int wma_parse_stream_properties(int fd, int size, MP3FILE *pmp3) {
 }
 
 /**
- * parse the audio media section... This is essentially a 
+ * parse the audio media section... This is essentially a
  * WAVFORMATEX structure.  Generally we only care about the
  * codec type.
  *
@@ -454,7 +460,9 @@ int wma_parse_audio_media(int fd, int size, MP3FILE *pmp3) {
         break;
     case 0x162:
         MAYBEFREE(pmp3->codectype);
-        pmp3->codectype = strdup("wmap"); /* pro */
+        MAYBEFREE(pmp3->type);
+        pmp3->codectype = strdup("wma"); /* pro */
+        pmp3->type = strdup("wmap");
         break;
     case 0x163:
         MAYBEFREE(pmp3->codectype);
@@ -510,9 +518,9 @@ int wma_parse_extended_content_description(int fd,int size, MP3FILE *pmp3, int e
         DPRINTF(E_DBG,L_SCAN,"Reading descr %d of %d\n",index,descriptor_count);
         if(!extended) {
             if(!wma_file_read_short(fd,&descriptor_name_len)) return FALSE;
-            if(!wma_file_read_utf16(fd,descriptor_name_len,&descriptor_name)) 
+            if(!wma_file_read_utf16(fd,descriptor_name_len,&descriptor_name))
                 return FALSE;
-            if(!wma_file_read_short(fd,&descriptor_value_type)) { 
+            if(!wma_file_read_short(fd,&descriptor_value_type)) {
                 free(descriptor_name);
                 return FALSE;
             }
@@ -527,7 +535,7 @@ int wma_parse_extended_content_description(int fd,int size, MP3FILE *pmp3, int e
             if(!wma_file_read_short(fd,&descriptor_name_len)) return FALSE;
             if(!wma_file_read_short(fd,&descriptor_value_type)) return FALSE;
             if(!wma_file_read_int(fd,&descriptor_value_int)) return FALSE;
-            if(!wma_file_read_utf16(fd,descriptor_name_len,&descriptor_name)) 
+            if(!wma_file_read_utf16(fd,descriptor_name_len,&descriptor_name))
                 return FALSE;
         }
 
@@ -627,7 +635,7 @@ int wma_parse_extended_content_description(int fd,int size, MP3FILE *pmp3, int e
                 pmp3->composer = descriptor_byte_value;
                 descriptor_byte_value = NULL;
             } else {
-                size = (int)strlen(pmp3->composer) + 1 + 
+                size = (int)strlen(pmp3->composer) + 1 +
                     (int)strlen(descriptor_byte_value) + 1;
                 tmp = malloc(size);
                 if(!tmp)
@@ -638,8 +646,8 @@ int wma_parse_extended_content_description(int fd,int size, MP3FILE *pmp3, int e
             }
         } else if(strcasecmp(descriptor_name,"wm/albumartist")==0) {
             /* get first one only */
-            if(!pmp3->orchestra) {
-                pmp3->orchestra = descriptor_byte_value;
+            if(!pmp3->album_artist) {
+                pmp3->album_artist = descriptor_byte_value;
                 descriptor_byte_value = NULL;
             }
         } else if(strcasecmp(descriptor_name,"author") == 0) {
@@ -712,7 +720,7 @@ int wma_parse_content_description(int fd,int size, MP3FILE *pmp3) {
                 return FALSE;
 
             DPRINTF(E_DBG,L_SCAN,"Got item of length %d: %s\n",sizes[index],utf8);
-            
+
             switch(index) {
             case 0: /* title */
                 if(pmp3->title)
@@ -725,7 +733,7 @@ int wma_parse_content_description(int fd,int size, MP3FILE *pmp3) {
                 pmp3->artist = utf8;
                 break;
             case 2: /* copyright - dontcare */
-                free(utf8);  
+                free(utf8);
             break;
             case 3: /* description */
                 if(pmp3->comment)
@@ -757,11 +765,11 @@ int wma_parse_file_properties(int fd,int size, MP3FILE *pmp3) {
     unsigned long long play_duration;
     unsigned long long send_duration;
     unsigned long long preroll;
-    
+
     unsigned int max_bitrate;
 
     /* skip guid (16 bytes), filesize (8), creation time (8),
-     * data packets (8) 
+     * data packets (8)
      */
     lseek(fd,40,SEEK_CUR);
 
@@ -783,7 +791,7 @@ int wma_parse_file_properties(int fd,int size, MP3FILE *pmp3) {
      * length.
      */
     pmp3->song_length = (int)((play_duration / 10000) - preroll);
-    
+
     /* skip flags(4),
      * min_packet_size (4), max_packet_size(4)
      */
@@ -801,7 +809,7 @@ int wma_parse_file_properties(int fd,int size, MP3FILE *pmp3) {
  * convert utf16 string to utf8.  This is a bit naive, but...
  * Since utf-8 can't expand past 4 bytes per code point, and
  * we're converting utf-16, we can't be more than 2n+1 bytes, so
- * we'll just allocate that much. 
+ * we'll just allocate that much.
  *
  * Probably it could be more efficiently calculated, but this will
  * always work.  Besides, these are small strings, and will be freed
@@ -959,7 +967,8 @@ int scan_get_wmainfo(char *filename, MP3FILE *pmp3) {
     int item;
     int err;
     int res=TRUE;
-    
+    int encrypted = 0;
+
     wma_fd = r_open2(filename,O_RDONLY);
     if(wma_fd == -1) {
         DPRINTF(E_INF,L_SCAN,"Error opening WMA file (%s): %s\n",filename,
@@ -993,7 +1002,7 @@ int scan_get_wmainfo(char *filename, MP3FILE *pmp3) {
     /* Now we just walk through all the headers and see if we
      * find anything interesting
      */
-    
+
     for(item=0; item < (int) hdr.objects; item++) {
         if(lseek(wma_fd,offset,SEEK_SET) == (off_t)-1) {
             DPRINTF(E_INF,L_SCAN,"Error seeking in %s\n",filename);
@@ -1027,6 +1036,8 @@ int scan_get_wmainfo(char *filename, MP3FILE *pmp3) {
                 res &= wma_parse_stream_properties(wma_fd,(int)subhdr.size,pmp3);
             } else if(strcmp(pguid->name,"ASF_Header_Extension_Object")==0) {
                 res &= wma_parse_header_extension(wma_fd,(int)subhdr.size,pmp3);
+            } else if(strstr(pguid->name,"Content_Encryption_Object")) {
+                encrypted=1;
             }
         } else {
             DPRINTF(E_DBG,L_SCAN,"Unknown subheader: %02hhx%02hhx%02hhx%02hhx-"
@@ -1054,6 +1065,14 @@ int scan_get_wmainfo(char *filename, MP3FILE *pmp3) {
     }
 
     r_close(wma_fd);
-    
+
+
+    if(encrypted) {
+        if(pmp3->codectype)
+            free(pmp3->codectype);
+
+        pmp3->codectype=strdup("wmap");
+    }
+
     return res;
 }
