@@ -1,5 +1,5 @@
 /*
- * $Id: scan-xml.c 1368 2006-08-27 00:02:09Z rpedde $
+ * $Id: scan-xml.c 1545 2007-04-18 03:34:03Z rpedde $
  * Implementation file iTunes metainfo scanning
  *
  * Copyright (C) 2005 Ron Pedde (ron@pedde.com)
@@ -25,6 +25,9 @@
 
 #include <ctype.h>
 #include <limits.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -58,7 +61,7 @@ static char *scan_xml_file; /** < The actual file we are scanning */
 static struct rbtree *scan_xml_db;
 
 #define MAYBECOPY(a) if(mp3.a) pmp3->a = mp3.a
-#define MAYBECOPYSTRING(a) if(mp3.a) { free(pmp3->a); pmp3->a = mp3.a; mp3.a=NULL; }
+#define MAYBECOPYSTRING(a) if(mp3.a) { if(pmp3->a) free(pmp3->a); pmp3->a = mp3.a; mp3.a=NULL; }
 #define MAYBEFREE(a) if((a)) { free((a)); (a)=NULL; }
 
 /** iTunes xml values we are interested in */
@@ -82,30 +85,34 @@ static char *scan_xml_track_tags[] = {
     "Location",
     "Date Added",
     "Comments",
+    "Composer",
+    "Album Artist",
     NULL
 };
 
 /** Indexes to the iTunes xml fields we are interested in */
-#define SCAN_XML_T_UNKNOWN     -1
-#define SCAN_XML_T_NAME         0
-#define SCAN_XML_T_ARTIST       1
-#define SCAN_XML_T_ALBUM        2
-#define SCAN_XML_T_GENRE        3
-#define SCAN_XML_T_TOTALTIME    4
-#define SCAN_XML_T_TRACKNUMBER  5
-#define SCAN_XML_T_TRACKCOUNT   6
-#define SCAN_XML_T_YEAR         7
-#define SCAN_XML_T_BITRATE      8
-#define SCAN_XML_T_SAMPLERATE   9
-#define SCAN_XML_T_PLAYCOUNT   10
-#define SCAN_XML_T_RATING      11
-#define SCAN_XML_T_DISABLED    12
-#define SCAN_XML_T_DISCNO      13
-#define SCAN_XML_T_DISCCOUNT   14
-#define SCAN_XML_T_COMPILATION 15
-#define SCAN_XML_T_LOCATION    16
-#define SCAN_XML_T_DATE_ADDED  17
-#define SCAN_XML_T_COMMENTS    18
+#define SCAN_XML_T_UNKNOWN      -1
+#define SCAN_XML_T_NAME          0
+#define SCAN_XML_T_ARTIST        1
+#define SCAN_XML_T_ALBUM         2
+#define SCAN_XML_T_GENRE         3
+#define SCAN_XML_T_TOTALTIME     4
+#define SCAN_XML_T_TRACKNUMBER   5
+#define SCAN_XML_T_TRACKCOUNT    6
+#define SCAN_XML_T_YEAR          7
+#define SCAN_XML_T_BITRATE       8
+#define SCAN_XML_T_SAMPLERATE    9
+#define SCAN_XML_T_PLAYCOUNT    10
+#define SCAN_XML_T_RATING       11
+#define SCAN_XML_T_DISABLED     12
+#define SCAN_XML_T_DISCNO       13
+#define SCAN_XML_T_DISCCOUNT    14
+#define SCAN_XML_T_COMPILATION  15
+#define SCAN_XML_T_LOCATION     16
+#define SCAN_XML_T_DATE_ADDED   17
+#define SCAN_XML_T_COMMENTS     18
+#define SCAN_XML_T_COMPOSER     19
+#define SCAN_XML_T_ALBUM_ARTIST 20
 
 #ifndef TRUE
 # define TRUE  1
@@ -216,7 +223,7 @@ int scan_xml_translate_path(char *pold, char *pnew) {
         ptemp++;
     }
     strcpy(working_path,pold);
-    
+
     if(!path_found) {
         DPRINTF(E_DBG,L_SCAN,"Translating %s, base %s\n",pold,scan_xml_file);
 
@@ -460,6 +467,12 @@ int scan_xml_playlist(char *filename) {
     }
 
     rbdestroy(scan_xml_db);
+
+    MAYBEFREE(scan_xml_itunes_version);
+    MAYBEFREE(scan_xml_itunes_base_path);
+    MAYBEFREE(scan_xml_itunes_decoded_base_path);
+    MAYBEFREE(scan_xml_real_base_path);
+
     return retval;
 }
 
@@ -686,7 +699,7 @@ int scan_xml_tracks_section(int action, char *info) {
             if((!is_streaming)&&scan_xml_translate_path(song_path,real_path)) {
                 /* FIXME: Error handling */
                 pmp3=db_fetch_path(NULL,real_path,0);
-                if(!pmp3) { 
+                if(!pmp3) {
                     /* file doesn't exist... let's add it? */
                     scan_filename(real_path,SCAN_TEST_COMPDIR,NULL);
                     pmp3=db_fetch_path(NULL,real_path,0);
@@ -700,6 +713,7 @@ int scan_xml_tracks_section(int action, char *info) {
                     MAYBECOPYSTRING(album);
                     MAYBECOPYSTRING(genre);
                     MAYBECOPYSTRING(comment);
+                    MAYBECOPYSTRING(composer);
                     MAYBECOPY(song_length);
                     MAYBECOPY(track);
                     MAYBECOPY(total_tracks);
@@ -712,6 +726,7 @@ int scan_xml_tracks_section(int action, char *info) {
                     MAYBECOPY(total_discs);
                     MAYBECOPY(time_added);
                     MAYBECOPY(disabled);
+                    MAYBECOPYSTRING(album_artist);
 
                     /* must add to the red-black tree */
                     scan_xml_add_lookup(current_track_id,pmp3->id);
@@ -728,7 +743,7 @@ int scan_xml_tracks_section(int action, char *info) {
                     DPRINTF(E_DBG,L_SCAN,"Adding %s\n",song_path);
                     pmp3 = calloc(sizeof(MP3FILE),1);
 
-                    if(!pmp3) 
+                    if(!pmp3)
                         DPRINTF(E_FATAL,L_SCAN,
                                 "malloc: scan_xml_tracks_section\n");
                 } else {
@@ -754,7 +769,8 @@ int scan_xml_tracks_section(int action, char *info) {
                 MAYBECOPY(rating);
                 MAYBECOPY(time_added);
                 MAYBECOPY(disabled);
-                    
+                MAYBECOPYSTRING(album_artist);
+
                 make_composite_tags(pmp3);
                 if(db_add(NULL,pmp3,&added_id) == DB_E_SUCCESS) {
                     scan_xml_add_lookup(current_track_id,added_id);
@@ -772,7 +788,9 @@ int scan_xml_tracks_section(int action, char *info) {
             MAYBEFREE(mp3.album);
             MAYBEFREE(mp3.genre);
             MAYBEFREE(mp3.comment);
+            MAYBEFREE(mp3.album_artist);
             MAYBEFREE(song_path);
+
             memset((void*)&mp3,0,sizeof(MP3FILE));
         } else {
             return XML_STATE_ERROR;
@@ -818,6 +836,10 @@ int scan_xml_tracks_section(int action, char *info) {
                 mp3.time_added = scan_xml_datedecode(info);
             } else if(current_field == SCAN_XML_T_COMMENTS) {
                 mp3.comment = strdup(info);
+            } else if(current_field == SCAN_XML_T_COMPOSER) {
+                mp3.composer = strdup(info);
+            } else if(current_field == SCAN_XML_T_ALBUM_ARTIST) {
+                mp3.album_artist = strdup(info);
             }
         } else if(action == RXML_EVT_END) {
             state = XML_TRACK_ST_TRACK_INFO;
@@ -917,8 +939,16 @@ int scan_xml_playlists_section(int action, char *info) {
             current_id=0;
             if(dont_scan == 0) {
                 DPRINTF(E_DBG,L_SCAN,"Creating playlist for %s\n",current_name);
+                /* we won't actually use the iTunes pl_id, as it seems
+                 *  to change for no good reason.  We'll hash the name,
+                 * instead. */
+
                 /* delete the old one first */
                 /* FIXME: Error handling */
+                DPRINTF(E_DBG,L_SCAN,"Converting native plid (%d) to %d\n",
+                        native_plid, util_djb_hash_str(current_name));
+                native_plid = util_djb_hash_str(current_name);
+
                 pm3u = db_fetch_playlist(NULL,scan_xml_file,native_plid);
                 if(pm3u) {
                     db_delete_playlist(NULL,pm3u->id);
@@ -931,9 +961,10 @@ int scan_xml_playlists_section(int action, char *info) {
                     DPRINTF(E_LOG,L_SCAN,"err adding playlist %s\n",current_name);
                     current_id=0;
                 }
-            } 
+            }
             dont_scan=0;
             state=XML_PL_ST_EXPECTING_PL_TRACKLIST;
+            MAYBEFREE(current_name);
             return XML_STATE_PLAYLISTS;
         }
         if(action == RXML_EVT_BEGIN)
@@ -945,8 +976,7 @@ int scan_xml_playlists_section(int action, char *info) {
         if(action == RXML_EVT_TEXT) {
             /* got the value we were hoping for */
             if(next_value == XML_PL_NEXT_VALUE_NAME) {
-                if(current_name)
-                    free(current_name);
+                MAYBEFREE(current_name);
                 current_name = strdup(info);
                 DPRINTF(E_DBG,L_SCAN,"Found playlist: %s\n",current_name);
                 /* disallow specific playlists */
