@@ -1,5 +1,5 @@
 /*
- * $Id: smart-parser.c 1353 2006-08-20 03:59:59Z rpedde $
+ * $Id: smart-parser.c 1622 2007-07-31 04:34:33Z rpedde $
  *
  * This is really two parts -- the lexer and the parser.  Converting
  * a parse tree back to a format that works with the database backend
@@ -273,6 +273,10 @@ FIELDLOOKUP sp_fields_0[] = {
     { T_INT_FIELD, "force_update", NULL },
     { T_STRING_FIELD, "codectype", NULL },
     { T_INT_FIELD, "idx", NULL },
+    { T_INT_FIELD, "has_video", NULL },
+    { T_INT_FIELD, "contentrating", NULL },
+    { T_INT_FIELD, "bits_per_sample", NULL },
+    { T_STRING_FIELD, "album_artist", NULL },
 
     /* end of db fields */
     { T_OR, "or", NULL },
